library("stringr")
library("Rsamtools")
library("Biostrings")
library("GenomicAlignments")
library("rtracklayer")
#library("rbamtools")
library("dplyr")

#### mutation_function.R

summation_split<-function(split,folder,bam_file,cds_start,cds_end){

	codon_num<-(cds_end-cds_start)/3
	codon_count<-NULL
	del_count<-NULL

	codon_64_count<-0
	aa_count<-0

	reads_depth<-matrix(0,nrow=(cds_end-cds_start),ncol=3)

	for(i in split){
	  
	  if(file.exists(paste0(folder,"/",bam_file,"_",i,".all.count.codon.txt"))){
		print(i)
		tmp_codon<-read.table(paste0(folder,"/",bam_file,"_",i,".all.count.codon.txt"),header=T,sep="\t",fill=TRUE,dec=".")
		codon_id<-paste0(tmp_codon[,1],"_",tmp_codon[,2])
		codon_count<-rbind(codon_count,cbind(codon_id,tmp_codon))
		
		tmp_del<-read.table(paste0(folder,"/",bam_file,"_",i,".ext.sort.inframe.del.count.txt"),header=F,skip=1,sep=",",fill=TRUE,dec=".")
		del_id<-paste0(tmp_del[,1],"_",tmp_del[,2])
		del_count<-rbind(del_count,cbind(del_id,tmp_del))
		
		
		tmp_64<-read.table(paste0(folder,"/",bam_file,"_",i,".subs.count.codon.csv"),header=T,row.names = 1,sep=",",fill=TRUE,dec=".")
		
		codon_64_count<-codon_64_count+tmp_64
		
		tmp_aa<-read.table(paste0(folder,"/",bam_file,"_",i,".subs.count.aa.csv"),header=T,row.names = 1,sep=",",fill=TRUE,dec=".")
		
		aa_count<-aa_count+tmp_aa
		
		reads_cover<-read.table(paste0(folder,"/",bam_file,"_",i,".coverage"),  header=FALSE,sep="\t",fill=TRUE,dec=".")
		reads_pos<-as.numeric(reads_cover[,2])
		reads_depth[,c(1:2)]<-as.matrix(reads_cover[which(reads_pos>cds_start & reads_pos<=cds_end),c(1:2)])
		reads_depth[,3]<-as.numeric(reads_depth[,3])+as.matrix(reads_cover[which(reads_pos>cds_start & reads_pos<=cds_end),3])
		

	  }
	}

	write.table(reads_depth, 
				paste0(folder,"/",bam_file,".coverage"), 
				append = FALSE, quote = FALSE, 
				sep = "\t",row.names = FALSE) 

	codon_num<-nrow(reads_depth)/3
	codon_depth<-rep(0,length(codon_num))

	for(c in 1:codon_num){
	  codon_start<-3*c-2
	  codon_end<-3*c
	  codon_depth[c]<-min(as.numeric(reads_depth[codon_start:codon_end,3]))
	}

	write.table(codon_depth, 
				paste0(folder,"/",bam_file,".coverage.codon.depth.txt"), 
				append = FALSE, quote = FALSE, 
				sep = "\t",row.names = FALSE) 
				
	colnames(codon_count)<-c("id","aa_pos","codon","count")

	codon_count_sum<-summarise(group_by(codon_count, id), 
							   aa_pos=unique(aa_pos),
							   codon=unique(codon),
							   count=sum(count))

	colnames(del_count)<-c("id","nt_pos","len","count")

	del_count_sum<-summarise(group_by(del_count, id), 
							 nt_pos=unique(nt_pos),
							 len=unique(len),
							 count=sum(count))

	write.table(del_count_sum[,-1], 
				paste0(folder,"/",bam_file,".ext.sort.inframe.del.count.txt"), 
				append = FALSE, quote = FALSE, 
				sep = ",",row.names = F) 

	write.table(codon_count_sum[,-1], 
				paste0(folder,"/",bam_file,".all.count.codon.txt"), 
				append = FALSE, quote = FALSE, 
				sep = "\t",row.names = FALSE) 


	subs_codon<-codon_64_count
	write.table(subs_codon, 
				paste0(folder,"/",bam_file,".subs.count.codon.csv"), 
				append = FALSE, quote = FALSE, 
				sep = ",",col.names = NA,
				row.names = TRUE) 	
	
	subs_codon_freq<-sweep(data.frame(subs_codon), 2, as.matrix(codon_depth), FUN = '/')
	write.table(subs_codon_freq, 
				paste0(folder,"/",bam_file,".subs.freq.codon.csv"), 
				append = FALSE, quote = FALSE, 
				sep = ",",col.names = NA,
				row.names = TRUE) 


	subs_aa<-aa_count
	write.table(subs_aa, 
				paste0(folder,"/",bam_file,".subs.count.aa.csv"), 
				append = FALSE, quote = FALSE, 
				sep = ",",col.names = NA,
				row.names = TRUE) 	
				
	subs_aa_freq<-sweep(data.frame(subs_aa), 2, as.matrix(codon_depth), FUN = '/')
	write.table(subs_aa_freq, 
				paste0(folder,"/",bam_file,".subs.freq.aa.csv"), 
				append = FALSE, quote = FALSE, 
				sep = ",",col.names = NA,
				row.names = TRUE) 


}


substitution_count_split<-function(cds_name,cds_start,cds_end,subs_bam_path,ref_path){
  
  print("substitution_count_split")
  
  indexBam(subs_bam_path)
  bamfile <- BamFile(subs_bam_path)
  
  tmp<-c("A","C","T","G")
  codon_list<-paste0(rep(tmp,16),rep(rep(tmp,each=4),4),
                     rep(tmp,each=16))
  codon_num<-(cds_end-cds_start)/3
  
  all_mut_codon<-NULL
  subs_codon<-matrix(0,ncol=codon_num,nrow=64)
  rownames(subs_codon)<-codon_list
  
  for(c in 1:codon_num){
    codon_start<-cds_start+(c-1)*3+1
    codon_end<-codon_start+2
    para<-paste0(cds_name,":",codon_start,"-",codon_end)
    
    codon_mut<-table(stackStringsFromBam(bamfile, param=para))
    mut_type<-names(codon_mut)
    subs_select<-which(mut_type%in%codon_list)
    
    all_mut_codon<-rbind(all_mut_codon,cbind(rep(c,length(codon_mut)),names(codon_mut),as.numeric(codon_mut)))
    
    
    subs_codon[mut_type[subs_select],c]<-codon_mut[subs_select]
	
	gc()
    
  }
  
  ref_seq<-readDNAStringSet(ref_path)
  cds_seq<-subseq(ref_seq, start=cds_start+1, end=cds_end)
  wt_codon<-as.character(codons(cds_seq[[1]]))
  
  for(pos in 1:length(wt_codon)){
    subs_codon[wt_codon[pos],pos]<-0
  }
  
  subs_aa<-codonToAA(subs_codon)
  #prex<-strsplit(subs_bam_path,split="[.]")[[1]][1]
  #prex<-subs_bam_path
  prex<-sub('\\.ext.sort.single.aa.subs.sort.bam$', '', subs_bam_path) 
  print(prex)
  write.table(all_mut_codon, 
              paste0(prex,".all.count.codon.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 
  
  write.table(subs_codon, 
              paste0(prex,".subs.count.codon.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  write.table(subs_aa, 
              paste0(prex,".subs.count.aa.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  
  gc()
}


del_count_split<-function(bam_path,cds_start,cds_end){
  
  indexBam(bam_path)
  
  print("del_count_split")
  
  alns <- GenomicAlignments::readGAlignments(file=bam_path,index=paste0(bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)

  reads_start<-start(alns)
  cigar_value<-cigar(alns)
  
  #  meta[i,2]<-length(cigar_value)
  
  del_type_num<-sapply(explodeCigarOps(cigar_value, ops=c("D")),length)
  
  cigar_table<-cigarOpTable(cigar_value)
  
  sum_del<-cigar_table[,"D"]
  sum_ins<-cigar_table[,"I"]
  
  inframe_del<-which(sum_ins==0 & del_type_num==1 & sum_del%%3==0)
  
  inframe_cigar_table<-cigar_table[inframe_del,]
  
  inframe_cigar<-cigar_value[inframe_del]
  
  reads_start_inframe_del<-reads_start[inframe_del]
  
  del_pos<-cigarRangesAlongReferenceSpace(inframe_cigar, 
                                            ops=c("D"), with.ops=TRUE)
  
  ### start(unlist(del_pos)) of first nt deletiton in reads is 1
  ### reads_start from 1
  inframe_del_start<-start(unlist(del_pos))+reads_start_inframe_del-1
  
  inframe_del_length<-width(unlist(del_pos))
  
  
  inframe_del_count<-table(paste0(inframe_del_start,",",
                                  inframe_del_length))
  
  write.table(cbind(inframe_del_count), 
              paste0(bam_path,".inframe.del.count.txt"), 
              append = FALSE, quote = FALSE, 
              sep = ",",row.names = TRUE) 
  
  #single_aa_del<-which(sum_ins==0 & del_type_num==1 & sum_del==3)	
  #export(alns[single_aa_del],paste0(bam_path,".single.aa.del.bam"))	
  #sortBam(paste0(bam_path,".single.aa.del.bam"), paste0(bam_path,".single.aa.del.sort"))  
  
  gc()
  
}


###### single_aa_del_count_split modified #######
single_aa_del_count_split<-function(single_aa_del_bam_path,cds_start,cds_end){

  indexBam(single_aa_del_bam_path)
  
  print("single_aa_del_count_split")
  
  bam_path<-single_aa_del_bam_path
  
  alns <- GenomicAlignments::readGAlignments(file=bam_path,index=paste0(bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)

  reads_start<-start(alns)
  cigar_value<-cigar(alns)
  
  del_type_num<-sapply(explodeCigarOps(cigar_value, ops=c("D")),length)
  
  cigar_table<-cigarOpTable(cigar_value)
  

  del_pos<-cigarRangesAlongReferenceSpace(cigar_value, 
                                            ops=c("D"), with.ops=TRUE)
  
  ### start(unlist(del_pos)) of first nt deletiton in reads is 1
  ### reads_start from 1
  single_aa_start<-start(unlist(del_pos))+reads_start-1
  
  
  single_aa_del_count<-table(paste0(single_aa_start,",",
                                  single_aa_length))
  
  write.table(cbind(inframe_del_count), 
              paste0(bam_path,".single.aa.del.count.txt"), 
              append = FALSE, quote = FALSE, 
              sep = ",",row.names = TRUE) 



}


extract_subs_mut_bam_loose<-function(mut_bam_path,geneid,cds_start,cds_end){

	print("extract_subs_mut_bam_loose")
	print(mut_bam_path)

	output_mut_tag<-paste0(mut_bam_path,".mut.tag.txt")

	gene_pos <- data.frame(seqid=geneid, start=cds_start, end=cds_end)
	
	indexBam(mut_bam_path)

	alns <- GenomicAlignments::readGAlignments(file=mut_bam_path,index=paste0(mut_bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)
	
	print("alns")

	if(length(which(names(alns)==""))>0){
	
		alns<-alns[-which(names(alns)=="")]
		
	}
	
    print("tags")
	tag_MD<-as.character(elementMetadata(alns)[,"MD"])
	tag_NM<-as.character(elementMetadata(alns)[,"NM"])
	tag_XM<-as.character(elementMetadata(alns)[,"XM"])
	cigar_value<-as.character(cigar(alns))
	start_pos<-as.numeric(start(alns))
	reads_len<-as.numeric(lengths(alns))
	
	write.table(c(mut_bam_path,length(start_pos)), 
			"../all_meta.txt", 
			append = TRUE, quote = FALSE, 
			sep = "\t",col.names = FALSE) 
			
	mut_matrix<-cbind(start_pos,cigar_value,
				  tag_MD,tag_NM,tag_XM,reads_len)

	write.table(mut_matrix, 
				output_mut_tag, 
				append = FALSE, quote = FALSE, 
				sep = "\t",col.names = FALSE) 


	
	subs_alns<-alns[which(tag_NM==tag_XM)]
	subs_MD<-as.character(elementMetadata(subs_alns)[,"MD"])
	subs_reads_start<-as.numeric(start(subs_alns))
	subs_reads_length<-as.numeric(lengths(subs_alns))
	
	export(subs_alns,paste0(mut_bam_path,".single.aa.subs.bam"))
	
	sortBam(paste0(mut_bam_path,".single.aa.subs.bam"), paste0(mut_bam_path,".single.aa.subs.sort"))
	

	gc()
				
}

extract_subs_mut_bam<-function(mut_bam_path,geneid,cds_start,cds_end){

	print("extract_subs_mut_bam")
	print(mut_bam_path)

	output_mut_tag<-paste0(mut_bam_path,".mut.tag.txt")

	gene_pos <- data.frame(seqid=geneid, start=cds_start, end=cds_end)
	
	indexBam(mut_bam_path)

	alns <- GenomicAlignments::readGAlignments(file=mut_bam_path,index=paste0(mut_bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)
	
	print("alns")

	if(length(which(names(alns)==""))>0){
	
		alns<-alns[-which(names(alns)=="")]
		
	}
	
    print("tags")
	tag_MD<-as.character(elementMetadata(alns)[,"MD"])
	tag_NM<-as.character(elementMetadata(alns)[,"NM"])
	tag_XM<-as.character(elementMetadata(alns)[,"XM"])
	cigar_value<-as.character(cigar(alns))
	start_pos<-as.numeric(start(alns))
	reads_len<-as.numeric(lengths(alns))
	
	write.table(c(mut_bam_path,length(start_pos)), 
			"../all_meta.txt", 
			append = TRUE, quote = FALSE, 
			sep = "\t",col.names = FALSE) 
			
	mut_matrix<-cbind(start_pos,cigar_value,
				  tag_MD,tag_NM,tag_XM,reads_len)

	write.table(mut_matrix, 
				output_mut_tag, 
				append = FALSE, quote = FALSE, 
				sep = "\t",col.names = FALSE) 


	
	subs_alns<-alns[which(tag_NM==tag_XM)]
	subs_MD<-as.character(elementMetadata(subs_alns)[,"MD"])
	subs_reads_start<-as.numeric(start(subs_alns))
	subs_reads_length<-as.numeric(lengths(subs_alns))

	subs_nt_count<-str_count(subs_MD, pattern = "[A-Z]")
	single_nt_subs<-which(subs_nt_count==1)
	multiple_nt_subs<-which(subs_nt_count>1 & subs_nt_count<4)

	get_subs_start_end_from_MD<-function(MD_tag){	
		tmp<-as.numeric(unlist(strsplit(MD_tag,split="[A-Z]")))
		c(tmp[1],tmp[length(tmp)])
	}
	
	print(length(multiple_nt_subs))
	
	if(length(multiple_nt_subs)>0){
	
		multiple_subs_distance<-matrix(unlist(lapply(subs_MD[multiple_nt_subs],get_subs_start_end_from_MD)),nrow=2)

		## cds_start from cds pos 0
		## nt substitution start from pos 0
		## reads start from pos 1
		first_nt_subs_aa_pos<-ceiling((subs_reads_start[multiple_nt_subs]+multiple_subs_distance[1,]-cds_start-1)/3)+1


		## multiple_subs_distance[2,] shows the count of matched nt from tails of reads
		## subs_reads_length-multiple_subs_distance[2,] is mismatch pos of reads from reads start
		## subs_reads_start+subs_reads_length-1 is pos of reads tail
		last_nt_subs_aa_pos<-ceiling((subs_reads_start[multiple_nt_subs]+subs_reads_length[multiple_nt_subs]-multiple_subs_distance[2,]-1-cds_start-1)/3)+1

		mulitple_subs_nt_single_aa<-multiple_nt_subs[which(first_nt_subs_aa_pos>0 & last_nt_subs_aa_pos>0 & first_nt_subs_aa_pos==last_nt_subs_aa_pos)]

		#subs_MD[mulitple_subs_nt_single_aa]

		single_aa_subs_bam<-subs_alns[c(single_nt_subs,mulitple_subs_nt_single_aa)]
		
	}else{
	
		single_aa_subs_bam<-subs_alns[single_nt_subs]
	}

    
	
	
	export(single_aa_subs_bam,paste0(mut_bam_path,".single.aa.subs.bam"))
	
	subs_reads_count<-length(c(single_nt_subs,mulitple_subs_nt_single_aa))
	
	if(subs_reads_count>100){
	
	  sortBam(paste0(mut_bam_path,".single.aa.subs.bam"), paste0(mut_bam_path,".single.aa.subs.sort"))
	  
	}else{
	  print(subs_reads_count)
	}
	

	gc()
				
}

codon_coverage<-function(coverage_file,cds_start,cds_end){

  print("codon_coverage")

  reads_cover<-read.table(coverage_file, 
                          header=FALSE,sep="\t",fill=TRUE,dec=".")
  reads_pos<-as.numeric(reads_cover[,2])
  reads_cover<-reads_cover[which(reads_pos>cds_start & reads_pos<=cds_end),]
  
  codon_num<-nrow(reads_cover)/3
  codon_depth<-rep(0,length(codon_num))
  
    for(c in 1:codon_num){
    codon_start<-3*c-2
    codon_end<-3*c
    
    codon_depth[c]<-min(reads_cover[codon_start:codon_end,3])
    
  }
  
    write.table(codon_depth, 
              paste0(coverage_file,".codon.depth.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 

}
##-------- discarded
reads_tag_cover<-function(bam_path,cds_start,cds_end){
  print("reads_tag_cover")
  
  cigar_path<- paste0(bam_path,".all.tag.txt")
  
  mut_file<-read.table(cigar_path, header=TRUE,sep="\t",fill=TRUE,dec=".")
  
  unique_pattern<-paste0(as.character(mut_file[,2]),
                         "_",as.character(mut_file[,3]),
                         "_",as.character(mut_file[,4]),
                         "_",as.character(mut_file[,5]),
                         "_",as.character(mut_file[,6]))
  
  write.table(table(unique_pattern), 
              paste0(bam_path,".all.table.tag.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 
  
  cover_pos<-paste0(as.character(mut_file[,2]),
                    "_",as.character(mut_file[,7]))
  
  write.table(table(cover_pos), 
              paste0(bam_path,".all.cover.pos.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 
  
  ##-----------
  
  codon_num<-(cds_end-cds_start)/3
  
  reads_cover<-read.table(paste0(bam_path,".all.cover.pos.txt"), 
                          header=TRUE,sep="\t",fill=TRUE,dec=".")
  
  reads_pos<-matrix(unlist(strsplit(as.character(reads_cover[,1]),"_")),
                    nrow=2)
  
  reads_start<-as.numeric(reads_pos[1,])
  reads_end<-reads_start+as.numeric(reads_pos[2,])
  reads_count<-as.numeric(reads_cover[,2])
  
  codon_depth<-rep(0,codon_num)
  
  for(c in 1:codon_num){
    codon_start<-cds_start+3*c-2
    codon_end<-cds_start+3*c
    
    sub_reads<-which(reads_start<=codon_start & reads_end>=codon_end)
    
    codon_depth[c]<-sum(reads_count[sub_reads])
    
  }
  
  
  write.table(codon_depth, 
              paste0(bam_path,".codon.depth.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 
  
}


substitution_freq<-function(cds_name,cds_start,cds_end,subs_bam_path,ref_path,codon_depth_path){
  
  print("substitution_freq")
  
  indexBam(subs_bam_path)
  bamfile <- BamFile(subs_bam_path)
  
  tmp<-c("A","C","T","G")
  codon_list<-paste0(rep(tmp,16),rep(rep(tmp,each=4),4),
                     rep(tmp,each=16))
  codon_num<-(cds_end-cds_start)/3
  
  all_mut_codon<-NULL
  subs_codon<-matrix(0,ncol=codon_num,nrow=64)
  rownames(subs_codon)<-codon_list
  
  for(c in 1:codon_num){
    codon_start<-cds_start+(c-1)*3+1
    codon_end<-codon_start+2
    para<-paste0(cds_name,":",codon_start,"-",codon_end)
    
    codon_mut<-table(stackStringsFromBam(bamfile, param=para))
    mut_type<-names(codon_mut)
    subs_select<-which(mut_type%in%codon_list)
    
    all_mut_codon<-rbind(all_mut_codon,cbind(rep(c,length(codon_mut)),names(codon_mut),as.numeric(codon_mut)))
    
    
    subs_codon[mut_type[subs_select],c]<-codon_mut[subs_select]
	
	gc()
    
  }
  
  ref_seq<-readDNAStringSet(ref_path)
  cds_seq<-subseq(ref_seq, start=cds_start+1, end=cds_end)
  wt_codon<-as.character(codons(cds_seq[[1]]))
  
  for(pos in 1:length(wt_codon)){
    subs_codon[wt_codon[pos],pos]<-0
  }
  
  subs_aa<-codonToAA(subs_codon)
  #prex<-strsplit(subs_bam_path,split="[.]")[[1]][1]
  #prex<-subs_bam_path
  prex<-sub('\\.ext.sort.single.aa.subs.sort.bam$', '', subs_bam_path) 
  print(prex)
  write.table(all_mut_codon, 
              paste0(prex,".all.count.codon.txt"), 
              append = FALSE, quote = FALSE, 
              sep = "\t",row.names = FALSE) 
  
  write.table(subs_codon, 
              paste0(prex,".subs.count.codon.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  write.table(subs_aa, 
              paste0(prex,".subs.count.aa.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  
  
  codon_depth<-read.table(codon_depth_path, header=TRUE,sep=",")
  
  subs_codon_freq<-sweep(data.frame(subs_codon), 2, as.matrix(codon_depth), FUN = '/')
  
  
  write.table(subs_codon_freq, 
              paste0(prex,".subs.freq.codon.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  
  
  subs_aa_freq<-sweep(data.frame(subs_aa), 2, as.matrix(codon_depth), FUN = '/')
  
  
  write.table(subs_aa_freq, 
              paste0(prex,".subs.freq.aa.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = NA,
              row.names = TRUE) 
  
  gc()
}


codonToAA<-function(codon_matrix){
  slc<-c("G","A","V","L","I","M","F","W","P","S","T","C","Y","N","Q","D","E","K","R","H","Stop")
  codon<-c(
  "GGT, GGC, GGA, GGG",  
  "GCT, GCC, GCA, GCG",
  "GTT, GTC, GTA, GTG",
  "CTT, CTC, CTA, CTG, TTA, TTG",
  "ATT, ATC, ATA",
  "ATG",
  "TTT, TTC",
  "TGG",
  "CCT, CCC, CCA, CCG",
  "TCT, TCC, TCA, TCG, AGT, AGC",
  "ACT, ACC, ACA, ACG",    
  "TGT, TGC",
  "TAT, TAC",
  "AAT, AAC",
  "CAA, CAG", 
  "GAT, GAC",
  "GAA, GAG",
  "AAA, AAG",
  "CGT, CGC, CGA, CGG, AGA, AGG",
  "CAT, CAC",
  "TAA, TAG, TGA")
  
  codon.list<-strsplit(codon,", ")
  
  aa_matrix<-matrix(0,ncol=ncol(codon_matrix),nrow=length(slc))
  rownames(aa_matrix)<-slc
  
  for(ss in 1:length(slc)){
    
    aa_matrix[ss,]<-colSums(codon_matrix[codon.list[[ss]],,drop=FALSE])
    
  }
  aa_matrix
}


del_freq<-function(bam_path,cds_start,cds_end,codon_depth_path){
  
  indexBam(bam_path)
  
  print("del_freq")
  
  alns <- GenomicAlignments::readGAlignments(file=bam_path,index=paste0(bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)

  reads_start<-start(alns)
  cigar_value<-cigar(alns)
  
  #  meta[i,2]<-length(cigar_value)
  
  del_type_num<-sapply(explodeCigarOps(cigar_value, ops=c("D")),length)
  
  cigar_table<-cigarOpTable(cigar_value)
  
  sum_del<-cigar_table[,"D"]
  sum_ins<-cigar_table[,"I"]
  
  inframe_del<-which(sum_ins==0 & del_type_num==1 & sum_del%%3==0)
  
  inframe_cigar_table<-cigar_table[inframe_del,]
  
  inframe_cigar<-cigar_value[inframe_del]
  
  reads_start_inframe_del<-reads_start[inframe_del]
  
  del_pos<-cigarRangesAlongReferenceSpace(inframe_cigar, 
                                          ops=c("D"), with.ops=TRUE)
  
  ### start(unlist(del_pos)) of first nt deletiton in reads is 1
  ### reads_start from 1
  inframe_del_start<-start(unlist(del_pos))+reads_start_inframe_del-1
  
  inframe_del_length<-width(unlist(del_pos))
  
  
  inframe_del_count<-table(paste0(inframe_del_start,",",
                                  inframe_del_length))
  
  write.table(cbind(inframe_del_count), 
              paste0(bam_path,".inframe.del.count.txt"), 
              append = FALSE, quote = FALSE, 
              sep = ",",row.names = TRUE) 
  
  ### cds_start is position 0 of cds
  cds_del<-which(inframe_del_start>cds_start & 
                   inframe_del_start<cds_end)
  inframe_del_start<-inframe_del_start[cds_del]
  inframe_del_length<-inframe_del_length[cds_del]
  
  ###-------
  aa_del_start<-floor((inframe_del_start-cds_start-1)/3)+1
  aa_del_length<-inframe_del_length/3
  
  codon_depth<-as.matrix(read.table(codon_depth_path, header=TRUE,sep=","))
  
  aa_del_count<-table(paste0(aa_del_start,",",
                             aa_del_length))
  aa_del_type<-names(aa_del_count)
  
  aa_del_freq<-matrix(0,ncol=4,nrow=length(aa_del_count))
  
  
  for(dd in 1:length(aa_del_count)){
    tmp<-strsplit(aa_del_type[dd],split=",")[[1]]
    del_start<-as.numeric(tmp[1])
    del_length<-as.numeric(tmp[2])
    del_count<-as.numeric(aa_del_count[dd])
    
    if((del_start+del_length)>length(codon_depth)){
      mean_coverage<-mean(codon_depth[c(del_start:length(codon_depth))])
    }else{
      mean_coverage<-mean(codon_depth[c(del_start:(del_start+del_length))])
    }
    
    
    del_freq<-del_count/mean_coverage
    
    aa_del_freq[dd,]<-c(del_start,del_length,del_count,del_freq)
  }
  
  colnames(aa_del_freq)<-c("del_start","del_length","del_count","del_freq")
  
  aa_del_freq<-aa_del_freq[order(aa_del_freq[,1]),]
  
  write.table(aa_del_freq, 
              paste0(bam_path,".aa.del.freq.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = T,
              row.names = F) 
			  
  gc()
  
}


del_freq_clean<-function(bam_path,cds_start,cds_end,codon_depth_path){
  
  indexBam(bam_path)
  
  print("del_freq")
  
  alns <- GenomicAlignments::readGAlignments(file=bam_path,index=paste0(bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)
  
  tag_XM<-as.character(elementMetadata(alns)[,"XM"])
  
  alns<-alns[which(tag_XM==0)]

  reads_start<-start(alns)
  cigar_value<-cigar(alns)
  
  
  #  meta[i,2]<-length(cigar_value)
  
  del_type_num<-sapply(explodeCigarOps(cigar_value, ops=c("D")),length)
  
  cigar_table<-cigarOpTable(cigar_value)
  
  sum_del<-cigar_table[,"D"]
  sum_ins<-cigar_table[,"I"]
  
  inframe_del<-which(sum_ins==0 & del_type_num==1 & sum_del%%3==0)
  
  inframe_cigar_table<-cigar_table[inframe_del,]
  
  inframe_cigar<-cigar_value[inframe_del]
  
  reads_start_inframe_del<-reads_start[inframe_del]
  
  del_pos<-cigarRangesAlongReferenceSpace(inframe_cigar, 
                                          ops=c("D"), with.ops=TRUE)
  
  ### start(unlist(del_pos)) of first nt deletiton in reads is 1
  ### reads_start from 1
  inframe_del_start<-start(unlist(del_pos))+reads_start_inframe_del-1
  
  inframe_del_length<-width(unlist(del_pos))
  
  
  inframe_del_count<-table(paste0(inframe_del_start,",",
                                  inframe_del_length))
  
  write.table(cbind(inframe_del_count), 
              paste0(bam_path,".inframe.del.count.txt"), 
              append = FALSE, quote = FALSE, 
              sep = ",",row.names = TRUE) 
  
  ### cds_start is position 0 of cds
  cds_del<-which(inframe_del_start>cds_start & 
                   inframe_del_start<cds_end)
  inframe_del_start<-inframe_del_start[cds_del]
  inframe_del_length<-inframe_del_length[cds_del]
  
  ###-------
  aa_del_start<-floor((inframe_del_start-cds_start-1)/3)+1
  aa_del_length<-inframe_del_length/3
  
  codon_depth<-as.matrix(read.table(codon_depth_path, header=TRUE,sep=","))
  
  aa_del_count<-table(paste0(aa_del_start,",",
                             aa_del_length))
  aa_del_type<-names(aa_del_count)
  
  aa_del_freq<-matrix(0,ncol=4,nrow=length(aa_del_count))
  
  
  for(dd in 1:length(aa_del_count)){
    tmp<-strsplit(aa_del_type[dd],split=",")[[1]]
    del_start<-as.numeric(tmp[1])
    del_length<-as.numeric(tmp[2])
    del_count<-as.numeric(aa_del_count[dd])
    
    if((del_start+del_length)>length(codon_depth)){
      mean_coverage<-mean(codon_depth[c(del_start:length(codon_depth))])
    }else{
      mean_coverage<-mean(codon_depth[c(del_start:(del_start+del_length))])
    }
    
    
    del_freq<-del_count/mean_coverage
    
    aa_del_freq[dd,]<-c(del_start,del_length,del_count,del_freq)
  }
  
  colnames(aa_del_freq)<-c("del_start","del_length","del_count","del_freq")
  
  aa_del_freq<-aa_del_freq[order(aa_del_freq[,1]),]
  
  write.table(aa_del_freq, 
              paste0(bam_path,".aa.del.freq.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = T,
              row.names = F) 
			  
  gc()
  
}



ins_freq<-function(bam_path,cds_start,cds_end,codon_depth_path){
  
  indexBam(bam_path)
  
  print("ins_freq")
  
  alns <- GenomicAlignments::readGAlignments(file=bam_path,index=paste0(bam_path,".bai") , param = Rsamtools::ScanBamParam(tag = c("MD","NM","XM"), what = c("seq","flag")), use.names = TRUE)

  reads_start<-start(alns)
  cigar_value<-cigar(alns)
  
  #  meta[i,2]<-length(cigar_value)
  
  ins_type_num<-sapply(explodeCigarOps(cigar_value, ops=c("I")),length)
  
  cigar_table<-cigarOpTable(cigar_value)
  
  sum_del<-cigar_table[,"D"]
  sum_ins<-cigar_table[,"I"]
  
  inframe_ins<-which(sum_del==0 & ins_type_num==1 & sum_ins%%3==0)
  
  inframe_cigar_table<-cigar_table[inframe_ins,]
  
  inframe_cigar<-cigar_value[inframe_ins]
  
  reads_start_inframe_ins<-reads_start[inframe_ins]
  
  ins_pos<-cigarRangesAlongReferenceSpace(inframe_cigar, 
                                          ops=c("I"), with.ops=TRUE)
  
  ### start(unlist(ins_pos)) of first nt insertion in reads is 1
  ### reads_start from 1
  inframe_ins_start<-start(unlist(ins_pos))+reads_start_inframe_ins-1
  
  inframe_ins_length<-sum_ins[inframe_ins]
  
  
  inframe_ins_count<-table(paste0(inframe_ins_start,",",
                                  inframe_ins_length))
  
  write.table(cbind(inframe_ins_count), 
              paste0(bam_path,".inframe.ins.count.txt"), 
              append = FALSE, quote = FALSE, 
              sep = ",",row.names = TRUE) 
  
  
  cds_ins<-which(inframe_ins_start>cds_start & 
                   inframe_ins_start<cds_end)
  inframe_ins_start<-inframe_ins_start[cds_ins]
  inframe_ins_length<-inframe_ins_length[cds_ins]
  
  ## cds_start is position of cds
  aa_ins_start<-floor((inframe_ins_start-cds_start-1)/3)+1
  aa_ins_length<-inframe_ins_length/3
  
  codon_depth<-as.matrix(read.table(codon_depth_path, header=TRUE,sep=","))
  
  aa_ins_count<-table(paste0(aa_ins_start,",",
                             aa_ins_length))
  aa_ins_type<-names(aa_ins_count)
  
  aa_ins_freq<-matrix(0,ncol=4,nrow=length(aa_ins_count))
  
  
  for(dd in 1:length(aa_ins_count)){
    tmp<-strsplit(aa_ins_type[dd],split=",")[[1]]
    ins_start<-as.numeric(tmp[1])
    ins_length<-as.numeric(tmp[2])
    ins_count<-as.numeric(aa_ins_count[dd])
    
    if((ins_start+ins_length)>length(codon_depth)){
      mean_coverage<-mean(codon_depth[c(ins_start:length(codon_depth))])
    }else{
      mean_coverage<-mean(codon_depth[c(ins_start:(ins_start+ins_length))])
    }
    
    
    ins_freq<-ins_count/mean_coverage
    
    aa_ins_freq[dd,]<-c(ins_start,ins_length,ins_count,ins_freq)
  }
  
  colnames(aa_ins_freq)<-c("ins_start","ins_length","ins_count","ins_freq")
  
  aa_ins_freq<-aa_ins_freq[order(aa_ins_freq[,1]),]
  
  write.table(aa_ins_freq, 
              paste0(bam_path,".aa.ins.freq.csv"), 
              append = FALSE, quote = FALSE, 
              sep = ",",col.names = T,
              row.names = F) 
			  
  gc()
  
}




