### result_plot_function.R

plot_cover_matrix_combine<-function(gene_list,del_matrix,combo_matrix,total_matrix,fig_path){
  cairo_pdf(fig_path, width = 6, height = 8)
  
    par(mfrow=c(3,1),mar=c(2,4,4,1))
	
	plot_cover_matrix(gene_list,total_matrix,"Total")
	plot_cover_matrix(gene_list,del_matrix,"Deletion")
	plot_cover_matrix(gene_list,combo_matrix,"Combo mutation")

	
  dev.off()

}

plot_cover_matrix<-function(gene_list,del_matrix,label){

  
	plot(c(1:10),del_matrix[,1],pch=20,type=c("o"),axes = FALSE,col=rainbow(6)[1],ylim=c(0,1),main=label,ylab="")

	for(g in 2:6){
	  points(c(1:10),del_matrix[,g],pch=20,type=c("o"),col=rainbow(6)[g],ylim=c(0,1))
	}
	axis(1, at=c(1:10), 
		 lab=paste0("\U2264",c(1:10)," AA"), las=TRUE)

	y<-c(0,0.2,0.4,0.6,0.8,1)

	axis(2, at=pretty(y), lab=paste0(pretty(y) * 100,"%"), las=TRUE)

	legend("bottomright",gene_list,
		   col=rainbow(6)[1:6],pch="-",border = NA, bty = 'n')




}



calc_freq<-function(count_path,cov_path,cds_start,cds_end){

  cut_count<-4

  ref1_del<-read.table(count_path,header=F,skip=1,sep=",",fill=TRUE,dec=".")
  
  tmp_pos_start<-as.numeric(ref1_del[,1])
  tmp_pos_end<-tmp_pos_start+as.numeric(ref1_del[,2])
  tmp_count<-as.numeric(ref1_del[,3])
  cds_check<-which(tmp_pos_start>cds_start & tmp_pos_end<cds_end & tmp_count>cut_count)
  
  ref1_del<-ref1_del[cds_check,]
  
  
  ref1_depth<-read.table(cov_path,header=F,sep="\t",fill=TRUE,dec=".")
  
  ref1_depth_pos<-as.character(ref1_depth$V2)
  ref1_cov<-as.numeric(ref1_depth[,3])
  print(mean(ref1_cov))
  ref1_del_count<-as.numeric(ref1_del[,3])
  
  ref1_del_freq<-rep(0,nrow(ref1_del))
  for(i in 1:nrow(ref1_del)){
    
    start_pos<-ref1_del[i,1]
    len<-ref1_del[i,2]
    
    cov_index<-which(ref1_depth_pos==start_pos)
    
    avg_cov<-mean(ref1_cov[cov_index:(cov_index+len-1)])
    
    ref1_del_freq[i]<-ref1_del_count[i]/avg_cov
    
    
  }
  
  
  data.frame(freq=ref1_del_freq,start=as.numeric(ref1_del[,1]),len=as.numeric(ref1_del[,2]))
  
  
}




plot_ref_wt_inframe_del_overlap<-function(folder,wt_label,ref1_label,ref2_label,cds_start,cds_end,fig_path,fig_path2){
  
  ref1_del_path<-paste0(folder,"/",ref1_label,".ext.sort.inframe.del.count.txt")

  ref1_cov_path<-paste0(folder,"/",ref1_label,".coverage")
  ref1_del<-calc_freq(ref1_del_path,ref1_cov_path,cds_start,cds_end)
  
  #ref1_del_freq<-ref1_del$freq
  
  ref2_del_path<-paste0(folder,"/",ref2_label,".ext.sort.inframe.del.count.txt")

  ref2_cov_path<-paste0(folder,"/",ref2_label,".coverage")
  ref2_del<-calc_freq(ref2_del_path,ref2_cov_path,cds_start,cds_end)
  
  all_del<-rbind(ref1_del,ref2_del)
  
  #length(unique(paste0(all_del$start,"_",all_del$len)))
  
  all_del_unique<-aggregate(freq~start+len,data=all_del, FUN=sum)
  
  del_aa_ref<-which((all_del_unique$start-cds_start)%%3==0)
  combo_aa_ref<-which((all_del_unique$start-cds_start)%%3>0)
  
  all_del_unique$freq<-all_del_unique$freq/2
  
  all_del_freq<-all_del_unique$freq
  
  wt_del_path<-paste0(folder,"/",wt_label,".ext.sort.inframe.del.count.txt")

  wt_cov_path<-paste0(folder,"/",wt_label,".coverage")
  wt_del<-calc_freq(wt_del_path,wt_cov_path,cds_start,cds_end)
  
  del_aa_wt<-which((wt_del$start-cds_start)%%3==0)
  combo_aa_wt<-which((wt_del$start-cds_start)%%3>0)
  
  wt_del_freq<-wt_del$freq
  
  hist_value1<-log10(all_del_freq[del_aa_ref]) 
  hist_value2<-log10(wt_del_freq[del_aa_wt])
 # hist_value3<-log10(wt_del_freq)
  #color1<-c(51,153,102)
  color1<-c(82,103,205)
  color2<-c(204,204,204)
  #fig_path<-output
  cairo_pdf(fig_path, width = 10, height = 8)
  
#  par(mfrow=c(2,1),mar=c(2,2,2,1))
  
  x_lower<-floor(min(c(hist_value1,hist_value2))*10)/10-0.1
  x_upper<-ceiling(max(c(hist_value1,hist_value2))*10)/10+0.1
  breaks_value<-seq(x_lower,x_upper,0.2)
  
  ylim_max<-floor(max(hist(hist_value1,breaks=breaks_value)$counts, hist(hist_value2,breaks=breaks_value)$counts)/50)*50+50
  
  hist1<-hist(hist_value1,breaks=breaks_value,col=rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",ylab=NULL,xlab=NULL,main=NULL)
#  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),ylim=c(0,ylim_max),border="white",xlab="log10 frequency",ylab=NULL,main="wild type")
  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",add=T,xlab="log10 frequency",ylab=NULL,main="wild type")
  
  legend("topright",
         c("reference","wild type"),
         cex=1.5,
         fill = c(rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),
                  rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255)),
         border = "white", bty = 'n')
  
  dev.off()
  
  
    hist_value1<-log10(all_del_freq[combo_aa_ref]) 
  hist_value2<-log10(wt_del_freq[combo_aa_wt])
 # hist_value3<-log10(wt_del_freq)
  #color1<-c(51,153,102)
  color1<-c(153,102,153)
  color2<-c(204,204,204)
  #fig_path<-output
  cairo_pdf(fig_path2, width = 10, height = 8)
  
#  par(mfrow=c(2,1),mar=c(2,2,2,1))
  
  x_lower<-floor(min(c(hist_value1,hist_value2))*10)/10-0.1
  x_upper<-ceiling(max(c(hist_value1,hist_value2))*10)/10+0.1
  breaks_value<-seq(x_lower,x_upper,0.2)
  
  ylim_max<-floor(max(hist(hist_value1,breaks=breaks_value)$counts, hist(hist_value2,breaks=breaks_value)$counts)/50)*50+50
  
  hist1<-hist(hist_value1,breaks=breaks_value,col=rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",ylab=NULL,xlab=NULL,main=NULL)
#  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),ylim=c(0,ylim_max),border="white",xlab="log10 frequency",ylab=NULL,main="wild type")
  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",add=T,xlab="log10 frequency",ylab=NULL,main="wild type")
  
  legend("topright",
         c("reference","wild type"),
         cex=1.5,
         fill = c(rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),
                  rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255)),
         border = "white", bty = 'n')
  
  dev.off()

  
  
  
#  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),add=T,border="white")
  
#  points(breaks_value[-length(breaks_value)],hist1$counts,pch=".",type=c("o"),col=rgb(color1[1],color1[2],color1[3], 255, maxColorValue=255), lwd=2)
  
#  points(breaks_value[-length(breaks_value)],hist2$counts,pch=".",type=c("o"),col=rgb(color2[1],color2[2],color2[3], 255, maxColorValue=255), lwd=2)
  
#  points(breaks_value[-length(breaks_value)],hist3$counts,pch=".",type=c("o"),col=rgb(color3[1],color3[2],color3[3], 255, maxColorValue=255), lwd=2)
  
#  curve(max(hist1$counts)*dnorm(x, mean=mean(hist_value1), sd=sd(hist_value1)),col=rgb(color1[1],color1[2],color1[3], 255, maxColorValue=255), lwd=2,add=TRUE)
  
#  curve(max(hist2$counts)*dnorm(x, mean=mean(hist_value2), sd=sd(hist_value2)), col=rgb(color2[1],color2[2],color2[3], 255, maxColorValue=255),lwd=2,add=TRUE)
  
#  curve(max(hist3$counts)*dnorm(x, mean=mean(hist_value3), sd=sd(hist_value3)),col=rgb(color3[1],color3[2],color3[3], 255, maxColorValue=255),lwd=2, add=TRUE)
  

  

  
  
}

plot_ref_wt_inframe_del_sep<-function(folder,wt_label,ref1_label,ref2_label,cds_start,cds_end,fig_path){
  
  ref1_del_path<-paste0(folder,"/",ref1_label,".ext.sort.inframe.del.count.txt")
  
  ref1_cov_path<-paste0(folder,"/",ref1_label,".coverage")
  ref1_del<-calc_freq(ref1_del_path,ref1_cov_path,cds_start,cds_end)
  
  #ref1_del_freq<-ref1_del$freq
  
  ref2_del_path<-paste0(folder,"/",ref2_label,".ext.sort.inframe.del.count.txt")
  
  ref2_cov_path<-paste0(folder,"/",ref2_label,".coverage")
  ref2_del<-calc_freq(ref2_del_path,ref2_cov_path,cds_start,cds_end)
  
  all_del<-rbind(ref1_del,ref2_del)
  
  #length(unique(paste0(all_del$start,"_",all_del$len)))
  
  all_del_unique<-aggregate(freq~start+len,data=all_del, FUN=sum)
  
  all_del_unique$freq<-all_del_unique$freq/2
  
  all_del_freq<-all_del_unique$freq
  
  wt_del_path<-paste0(folder,"/",wt_label,".ext.sort.inframe.del.count.txt")
  
  wt_cov_path<-paste0(folder,"/",wt_label,".coverage")
  wt_del<-calc_freq(wt_del_path,wt_cov_path,cds_start,cds_end)
  wt_del_freq<-wt_del$freq
  
  hist_value1<-log10(all_del_freq) 
  hist_value2<-log10(wt_del_freq)
  # hist_value3<-log10(wt_del_freq)
  
  #color1<-c(51,153,102)
  color1<-c(153,102,153)
  color2<-c(204,204,204)
  #fig_path<-output
  cairo_pdf(fig_path, width = 10, height = 8)
  
  par(mfrow=c(2,1),mar=c(2,2,2,1))
  
  x_lower<-floor(min(c(hist_value1,hist_value2))*10)/10-0.1
  x_upper<-ceiling(max(c(hist_value1,hist_value2))*10)/10+0.1
  breaks_value<-seq(x_lower,x_upper,0.2)
  
  ylim_max<-floor(max(hist(hist_value1,breaks=breaks_value)$counts, hist(hist_value2,breaks=breaks_value)$counts)/50)*50+50
  
  hist1<-hist(hist_value1,breaks=breaks_value,col=rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",ylab=NULL,xlab=NULL,main="reference")
  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",xlab="log10 frequency",ylab=NULL,main="wild type")
  
  dev.off()
  #  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),add=T,border="white")
  
  #  points(breaks_value[-length(breaks_value)],hist1$counts,pch=".",type=c("o"),col=rgb(color1[1],color1[2],color1[3], 255, maxColorValue=255), lwd=2)
  
  #  points(breaks_value[-length(breaks_value)],hist2$counts,pch=".",type=c("o"),col=rgb(color2[1],color2[2],color2[3], 255, maxColorValue=255), lwd=2)
  
  #  points(breaks_value[-length(breaks_value)],hist3$counts,pch=".",type=c("o"),col=rgb(color3[1],color3[2],color3[3], 255, maxColorValue=255), lwd=2)
  
  #  curve(max(hist1$counts)*dnorm(x, mean=mean(hist_value1), sd=sd(hist_value1)),col=rgb(color1[1],color1[2],color1[3], 255, maxColorValue=255), lwd=2,add=TRUE)
  
  #  curve(max(hist2$counts)*dnorm(x, mean=mean(hist_value2), sd=sd(hist_value2)), col=rgb(color2[1],color2[2],color2[3], 255, maxColorValue=255),lwd=2,add=TRUE)
  
  #  curve(max(hist3$counts)*dnorm(x, mean=mean(hist_value3), sd=sd(hist_value3)),col=rgb(color3[1],color3[2],color3[3], 255, maxColorValue=255),lwd=2, add=TRUE)
  
  #  legend("topright",
  #         c("reference","wild type"),
  #         cex=1.5,
  #         fill = c(rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),
  ##                  rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255)),
  #         border = "white", bty = 'n')
  
  
  
  
}

plot_ref_wt_aa_substitution_sep<-function(folder,wt_label,ref1_label,ref2_label,fig_path){
  
  ref1_subs_path<-paste0(folder,"/",ref1_label,".subs.freq.aa.csv")
  
  ref1_subs<-read.table(ref1_subs_path,header=T,sep=",",fill=TRUE,dec=".")

  ref2_subs_path<-paste0(folder,"/",ref2_label,".subs.freq.aa.csv")
  
  ref2_subs<-read.table(ref2_subs_path,header=T,sep=",",fill=TRUE,dec=".")
  
  identical(ref1_subs[,1],ref2_subs[,1])
  
  all_subs<-c(as.matrix((ref1_subs[,-1]+ref2_subs[,-1])/2))
  all_subs<-all_subs[which(all_subs>0)]
  
  wt_subs_path<-paste0(folder,"/",wt_label,".subs.freq.aa.csv")
  wt_subs<-read.table(wt_subs_path,header=T,sep=",",fill=TRUE,dec=".")

  wt_subs<-c(as.matrix(wt_subs[,-1]))
  wt_subs<-wt_subs[which(wt_subs>0)]
  
  
  hist_value1<-log10(all_subs) 
  hist_value2<-log10(wt_subs)
  # hist_value3<-log10(wt_del_freq)
  
  color1<-c(51,153,102)
  #color1<-c(153,102,153)
  color2<-c(204,204,204)
  #fig_path<-output
  cairo_pdf(fig_path, width = 10, height = 8)
  
  par(mfrow=c(2,1),mar=c(2,2,2,1))
  
  x_lower<-floor(min(c(hist_value1,hist_value2))*10)/10-0.1
  x_upper<-ceiling(max(c(hist_value1,hist_value2))*10)/10+0.1
  
  breaks_value<-seq(x_lower,x_upper,0.2)
  #print(breaks_value)
  
  #breaks_value<-seq(-7,-1,0.2)
  ylim_max<-floor(max(hist(hist_value1,breaks=breaks_value)$counts, hist(hist_value2,breaks=breaks_value)$counts)/50)*50+50
  
  hist1<-hist(hist_value1,breaks=breaks_value,col=rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",ylab=NULL,xlab=NULL,main="reference")
  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",xlab="log10 frequency",ylab=NULL,main="wild type")
  
  dev.off()
  
  
  
  
  
  
}



plot_ref_wt_aa_substitution_overlap<-function(folder,wt_label,ref1_label,ref2_label,fig_path){
  
  ref1_subs_path<-paste0(folder,"/",ref1_label,".subs.freq.aa.csv")
  
  ref1_subs<-read.table(ref1_subs_path,header=T,sep=",",fill=TRUE,dec=".")
  
  ref2_subs_path<-paste0(folder,"/",ref2_label,".subs.freq.aa.csv")
  
  ref2_subs<-read.table(ref2_subs_path,header=T,sep=",",fill=TRUE,dec=".")
  
  identical(ref1_subs[,1],ref2_subs[,1])
  
  all_subs<-c(as.matrix((ref1_subs[,-1]+ref2_subs[,-1])/2))
  all_subs<-all_subs[which(all_subs>0)]
  
  wt_subs_path<-paste0(folder,"/",wt_label,".subs.freq.aa.csv")
  wt_subs<-read.table(wt_subs_path,header=T,sep=",",fill=TRUE,dec=".")
  
  wt_subs<-c(as.matrix(wt_subs[,-1]))
  wt_subs<-wt_subs[which(wt_subs>0)]
  
  
  hist_value1<-log10(all_subs) 
  hist_value2<-log10(wt_subs)
  # hist_value3<-log10(wt_del_freq)
  
  color1<-c(51,153,102)
  #color1<-c(153,102,153)
  color2<-c(204,204,204)
  #fig_path<-output
  cairo_pdf(fig_path, width = 10, height = 8)
  
  x_lower<-floor(min(c(hist_value1,hist_value2))*10)/10-0.1
  x_upper<-ceiling(max(c(hist_value1,hist_value2))*10)/10+0.1
  
  breaks_value<-seq(x_lower,x_upper,0.2)
  #print(breaks_value)
  #par(mfrow=c(2,1),mar=c(2,2,2,1))
  
  #breaks_value<-seq(min(c(hist_value1,hist_value2))-0.1,-1,0.2)
  
  #breaks_value<-seq(-7,-1,0.2)
  ylim_max<-floor(max(hist(hist_value1,breaks=breaks_value)$counts, hist(hist_value2,breaks=breaks_value)$counts)/50)*50+50
  
  hist1<-hist(hist_value1,breaks=breaks_value,col=rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",ylab=NULL,xlab=NULL,main=NULL)
  hist2<-hist(hist_value2,breaks=breaks_value,col=rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255),xlim=c(floor(x_lower),ceiling(x_upper)),ylim=c(0,ylim_max),border="white",xlab="log10 frequency",add=T,ylab=NULL)
  
  legend("topright",
         c("reference","wild type"),
         cex=1.5,
         fill = c(rgb(color1[1],color1[2],color1[3], 150, maxColorValue=255),
                  rgb(color2[1],color2[2],color2[3], 150, maxColorValue=255)),
         border = "white", bty = 'n')
  
  
  dev.off()
  
}




plot_ref_wt_indel<-function(folder,wt_label,ref1_label,ref2_label,cds_start,cds_end){
  
  ref1_del_path<-paste0(folder,"/",ref1_label,".ext.sort.del.all.count.txt")
  ref1_ins_path<-paste0(folder,"/",ref1_label,".ext.sort.ins.all.count.txt")
  ref1_cov_path<-paste0(folder,"/",ref1_label,".coverage")
  ref1_del_freq<-calc_freq(ref1_del_path,ref1_cov_path,cds_start,cds_end)
  ref1_ins_freq<-calc_freq(ref1_ins_path,ref1_cov_path,cds_start,cds_end)
  
  ref2_del_path<-paste0(folder,"/",ref2_label,".ext.sort.del.all.count.txt")
  ref2_ins_path<-paste0(folder,"/",ref2_label,".ext.sort.ins.all.count.txt")
  ref2_cov_path<-paste0(folder,"/",ref2_label,".coverage")
  ref2_del_freq<-calc_freq(ref2_del_path,ref2_cov_path,cds_start,cds_end)
  ref2_ins_freq<-calc_freq(ref2_ins_path,ref2_cov_path,cds_start,cds_end)
  
  wt_del_path<-paste0(folder,"/",wt_label,".ext.sort.del.all.count.txt")
  wt_ins_path<-paste0(folder,"/",wt_label,".ext.sort.ins.all.count.txt")
  wt_cov_path<-paste0(folder,"/",wt_label,".coverage")
  wt_del_freq<-calc_freq(wt_del_path,wt_cov_path,cds_start,cds_end)
  wt_ins_freq<-calc_freq(wt_ins_path,wt_cov_path,cds_start,cds_end)
  
  
  hist(log10(c(ref1_del_freq,ref1_ins_freq)),breaks=30,col=rgb(0, 0, 255, 60, maxColorValue=255),ylim=c(0,400))
  
  hist(log10(c(ref2_del_freq,ref2_ins_freq)),breaks=30,col=rgb(0, 255, 255, 60, maxColorValue=255),add=T)
  
  hist(log10(c(wt_del_freq,wt_ins_freq)),breaks=30,col=rgb(255, 255, 0, 60, maxColorValue=255),add=T)
  
  
  
}



plot_del_score_for_ds<-function(del_score,fig_path,red_p,black_p,WT_aa){

aa_len<-length(WT_aa)
	if(aa_len<300){
		axis_break<-20
	}else if(aa_len<700){
		axis_break<-100
	}else if(aa_len<700){
		axis_break<-60
	}else if(aa_len<1000){
		axis_break<-100
	}else{
		axis_break<-200
	}

	y_axis_lim<-ceiling(max(del_score[,2])*1.1)
    y_break<-max(floor(y_axis_lim/4),1)
	#print(y_axis_lim)
	#png(fig_path, width = 1200, height = 300)
    cairo_pdf(fig_path, width = 16, height = 4)
	plot(del_score[,1],del_score[,2],pch=16,col="grey",xpd=FALSE,axes = FALSE,
	#xlim=c(0,aa_len),
		 ylim=c(0,y_axis_lim),
		 #xlim=c(0,axis_break*round(aa_len/axis_break,0)),
		 xlim=c(0,axis_break*ceiling(aa_len/axis_break)),
		 xaxs = "i",yaxs = "i",ann=FALSE)
	abline(h = 1, lty=2)
	axis(side=1,at=seq(0,axis_break*ceiling(aa_len/axis_break),axis_break),tck=-0.02)
	#axis(side=1,at=seq(0,aa_len,axis_break))
	if(y_axis_lim>4){
	axis(side=2,at=seq(0,y_axis_lim,y_break),tck=-0.02)
	}else{
	axis(side=2,at=seq(0,y_axis_lim,y_break),tck=-0.02)
	}
	

	########
	
	if(length(black_p)>0){

	points_index<-black_p
	points(del_score[points_index,1:2],col="black",pch=16)
	#with(del_score[points_index,1:2],text(del_score[points_index,1:2],labels=paste0(WT_aa[black_p],black_p),pos=c(3,4,1)))

}
	if(length(red_p)>0){
		points_index<-red_p
		points(del_score[points_index,1:2],col="red",pch=16)
		#with(del_score[points_index,1:2],text(del_score[points_index,1:2],labels=paste0(WT_aa[red_p],red_p),pos=c(2,3,4)))
	}



	dev.off()



}




plot_subs_freq<-function(color,aa_len,avg_sum_freq,fig_path){

	bar_width<-0.85

	if(aa_len<300){
		axis_break<-20
	}else if(aa_len<700){
		axis_break<-100
	}else if(aa_len<700){
		axis_break<-60
	}else if(aa_len<1000){
		axis_break<-100
	}else{
		axis_break<-200
	}
	  axis_lim<-axis_break*ceiling(aa_len/axis_break)
	  space_value<-(1-bar_width)/bar_width
	  #space_value<-0.182
  #fig_path<-paste0(figFolder,"/",gene_label,"_subs_freq.pdf")
#  png(fig_path, width = 6*aa_len, height = 2*aa_len)

avg_sum_freq_log<-log10(avg_sum_freq+1e-05)
cut_subs<-mean(avg_sum_freq_log)+3*sd(avg_sum_freq_log)
avg_sum_freq[avg_sum_freq_log<cut_subs]<-0

  cairo_pdf(fig_path, width = 16, height = 4)
  #cairo_pdf(fig_path, width = 10, height = 8)   
  tmp<-ceiling(max(avg_sum_freq)/0.05)
  
  barplot(avg_sum_freq,ylim=c(0,tmp*0.05),xpd=FALSE,tcl = NA,  
          border="NA",xaxs = "i",
		  col=rgb(color[1], color[2], color[3], 255, maxColorValue=255),
		  width = bar_width,xlim=c(-0.5,axis_lim),
          space=space_value,cex.axis=1)
  
  axis(side=1,at=seq(-0.5,axis_lim,axis_break),label=seq(0,axis_lim,axis_break),tck=-0.02)
  dev.off()
  




}


test_plot_del_lfc_transp<-function(color1,color2,del_lfc,aa_len,del_cut,del_len_cut,del_type){



para_lwd<-1
para_cex_lab<-1
para_cex_axis<-1

bar_width<-0.85
para_space<-(1-bar_width)/bar_width
#para_space<-0

  	if(aa_len<300){
		axis_break<-20
	}else{
		axis_break<-100
	}
	#axis_lim<-aa_len
	axis_lim<-axis_break*ceiling(aa_len/axis_break)

	#print(y_lim)
  #print(which(del_lfc[,"single_aa_del"]>del_cut))
  #single_aa_del<-del_lfc[,"single_aa_del"]
  single_aa_del<-del_lfc[,1]
  if(del_len_cut>0){
    y_lim<-ceiling(max(del_lfc[,c(1:del_len_cut)],single_aa_del))
  }else{
    y_lim<-ceiling(max(single_aa_del))
  }

  if(del_len_cut>0){
  
	del_lfc<-del_lfc[,c(1:del_len_cut),drop=FALSE]
	flag<-1
	  for(aa in 1:nrow(del_lfc)){
		del_len<-which(del_lfc[aa,]>0)
		
		if(length(del_len)>0){
		  #print(aa)
		  
		  for(ii in 1:length(del_len)){
			bar<-rep(0,aa_len)
			#print(ii)
			
			p<-del_len[ii]

			if(del_type==1){
			  #bar_col<-rgb(0, 0, 255, 180/p, maxColorValue=255)
			  bar_col<-rgb(color1[1], color1[2], color1[3], 25, maxColorValue=255)
			  label<-paste0(color1[1],",", color1[2],",", color1[3])
			}else{
			#bar_col<-rgb(128,0,128, 180/p, maxColorValue=255)
			  bar_col<-rgb(color2[1], color2[2], color2[3], 255/(p+1), maxColorValue=255)
			  label<-paste0(color2[1],",", color2[2],",", color2[3])
			}
			
			if(del_type==1){
			  bar[aa:c(aa+p-1)]<-del_lfc[aa,p]
			}else{
			  bar[aa:c(aa+p)]<-del_lfc[aa,p]
			}
			
			
			
			
			if(flag==1){

			### 3nt deletion cover two aa
#print("in")
			barplot(bar,space=para_space,width=bar_width,
xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),
			border=NA,tcl = NA,xaxs = "i",yaxs = "i",
			xpd=FALSE,beside=TRUE,main=label,
			col=bar_col)

			flag<-0
			
			}else{
			### bar_length is del_length
			#bar[aa:c(aa+p-1)]<-del_lfc[aa,p]

			barplot(bar,space=para_space,width=bar_width,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,
			xpd=FALSE,beside=TRUE,
			col=bar_col)
			}

		  }
		  
		}
		
	  }
  
  
  
  }
  
if(del_type==1){
  bar_col<-rgb(color1[1], color1[2], color1[3], 255, maxColorValue=255)

	if(del_len_cut==0){
		barplot(single_aa_del,space=para_space,width=bar_width,
		xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),tcl = NA,
			  border=NA,col=bar_col,xaxs = "i",axes=F,
			  xpd=FALSE,beside=TRUE,xlab="",ylab="")
		
	}else{
		barplot(single_aa_del,space=para_space,width=bar_width,
		add=T,tcl = NA,xaxs = "i",axes=F,
			  border=NA,col=bar_col,
			  xpd=FALSE,beside=TRUE,xlab="",ylab="")

	}


}

axis(side=1,
		at=seq(0,axis_lim,axis_break),tck=-0.02)
  

}




test_plot_del_bar_inner<-function(color1,color2,aa_len,del_lfc1,del_lfc2,y_lim,del_cut){

	para_lwd<-1
	para_cex_lab<-1
	para_cex_axis<-1

	bar_width<-0.85
	para_space<-(1-bar_width)/bar_width

	#para_space<-0

	if(aa_len<300){
		axis_break<-20
	}else{
		axis_break<-100
	}

	axis_lim<-axis_break*ceiling(aa_len/axis_break)

	for(aa in 1:nrow(del_lfc1)){
	
	
	
	
	del_len2<-which(del_lfc2[aa,]>0)

	if(length(del_len2)>0){
	  for(ii in 1:length(del_len2)){
	  	p<-del_len2[ii]
		bar2<-rep(0,aa_len)		
		bar_col2<-rgb(color2[1],color2[2],color2[3],240/p, maxColorValue=255)
		bar2[aa:c(aa+p)]<-del_lfc2[aa,p]
			
		### bar_length is del_length

		barplot(bar2,space=para_space,add=T,tcl = NA,
		border=NA,xaxs = "i",axes=F,width=bar_width,
		xpd=FALSE,beside=TRUE,
		col=bar_col2)
		
	  }### for del_len		  
	}## del_len if	

	del_len1<-which(del_lfc1[aa,]>0)

	if(length(del_len1)>0){
	  for(ii in 1:length(del_len1)){
		bar1<-rep(0,aa_len)
		#print(ii)
		p<-del_len1[ii]
		
		if(p>1){
			bar_col1<-rgb(color1[1],color1[2],color1[3],240/p, maxColorValue=255)
			bar1[aa:c(aa+p-1)]<-del_lfc1[aa,p]
		
			### bar_length is del_length
			barplot(bar1,space=para_space,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,width=bar_width,
			xpd=FALSE,beside=TRUE,
			col=bar_col1)
			
		} ## p if		
	  }### for del_len		  
	}## del_len if	

	} ## for aa position 
   
}


plot_del_bar_inner<-function(aa_len,del_lfc1,del_lfc2,y_lim,del_cut){

	para_lwd<-1
	para_cex_lab<-1
	para_cex_axis<-1

	bar_width<-0.85
	para_space<-(1-bar_width)/bar_width

	#para_space<-0

	if(aa_len<300){
		axis_break<-20
	}else{
		axis_break<-100
	}

	axis_lim<-axis_break*ceiling(aa_len/axis_break)

	for(aa in 1:nrow(del_lfc1)){
	
	
	
	
	del_len2<-which(del_lfc2[aa,]>0)

	if(length(del_len2)>0){
	  for(ii in 1:length(del_len2)){
	  	p<-del_len2[ii]
		bar2<-rep(0,aa_len)		
		bar_col2<-rgb(255,165,0 ,180/p, maxColorValue=255)
		bar2[aa:c(aa+p)]<-del_lfc2[aa,p]
			
		### bar_length is del_length

		barplot(bar2,space=para_space,add=T,tcl = NA,
		border=NA,xaxs = "i",axes=F,width=bar_width,
		xpd=FALSE,beside=TRUE,
		col=bar_col2)
		
	  }### for del_len		  
	}## del_len if	

	del_len1<-which(del_lfc1[aa,]>0)

	if(length(del_len1)>0){
	  for(ii in 1:length(del_len1)){
		bar1<-rep(0,aa_len)
		#print(ii)
		p<-del_len1[ii]
		
		if(p>1){
			bar_col1<-rgb(0,0,255,180/p, maxColorValue=255)
			bar1[aa:c(aa+p-1)]<-del_lfc1[aa,p]
		
			### bar_length is del_length
			barplot(bar1,space=para_space,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,width=bar_width,
			xpd=FALSE,beside=TRUE,
			col=bar_col1)
			
		} ## p if		
	  }### for del_len		  
	}## del_len if	

	} ## for aa position 
   
}


## plot deletion 
plot_del_lfc_transp_double<-function(del_lfc,aa_len,del_cut,del_len_cut){

para_lwd<-1
para_cex_lab<-1
para_cex_axis<-1

bar_width<-0.85
para_space<-(1-bar_width)/bar_width

if(aa_len<300){
	axis_break<-20
}else{
	axis_break<-100
}

axis_lim<-axis_break*ceiling(aa_len/axis_break)

single_aa_del<-del_lfc[,1]

if(del_len_cut>0){
	y_lim<-ceiling(max(del_lfc,single_aa_del))
	}else{
	y_lim<-ceiling(max(single_aa_del))
}

#print(y_lim)
### plot blank
barplot(rep(0,aa_len),space=para_space,tcl = NA,width=bar_width,
xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),
	  border=NA,xaxs = "i",
	  xpd=FALSE,beside=TRUE,xlab="",ylab="")
axis(side=1,at=seq(0,axis_lim,axis_break),tck=-0.02)
   
bar_col1<-rgb(0, 0, 255, 180, maxColorValue=255)
barplot(single_aa_del,space=para_space,tcl = NA,width=bar_width,
xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),add=T,
	  border=NA,col=bar_col1,xaxs = "i",
	  xpd=FALSE,beside=TRUE,xlab="",ylab="")

if(del_len_cut>0){

	del_lfc1<-del_lfc[,c(1:del_len_cut),drop=FALSE]
	del_lfc2<-del_lfc[,c(11:(10+del_len_cut)),drop=FALSE]
	#flag<-1
	plot_del_bar_inner(aa_len,del_lfc1,del_lfc2,y_lim,del_cut)
	#plot_del_bar_inner(aa_len,del_lfc2,y_lim,del_cut,0,0)

}### del_len_cut if
  



  

#  legend("topright",
#         c("single aa deletion","2 aa deletion",
#		 "3 aa deletion","4 aa deletion","5 aa deletion",
#		 "6 aa deletion","7 aa deletion","8 aa deletion",
#		 "9 aa deletion"),
#         cex=0.8,
#         fill = c("darkblue",
#                  rgb(0, 0, 255, 255/2, maxColorValue=255),
#				  rgb(0, 0, 255, 255/3, maxColorValue=255),
#				  rgb(0, 0, 255, 255/4, maxColorValue=255),
#				  rgb(0, 0, 255, 255/5, maxColorValue=255),
#				  rgb(0, 0, 255, 255/6, maxColorValue=255),
#				  rgb(0, 0, 255, 255/7, maxColorValue=255),
#				  rgb(0, 0, 255, 255/8, maxColorValue=255),
#				  rgb(0, 0, 255, 255/9, maxColorValue=255)),
#				  border = NA, bty = 'n')
  

}



subs_shift_del<-function(figFolder,tableFolder,gene_label,shift_len,y_lim,del_fc,aa_len,WT_aa,red_p){

	if(aa_len<300){
		axis_break<-20
		bar_width<-0.6
	}else{
		axis_break<-100
		bar_width<-0.9
	}
	space_value<-(1-bar_width)/bar_width
	axis_lim<-axis_break*round(aa_len/axis_break,0)
#	fig_path<-paste0(figFolder,"/",gene_label,"_shift_",shift_len,"_subs_fc.png")
#	png(fig_path, width = 1200, height = 400)
	fig_path<-paste0(figFolder,"/",gene_label,"_shift_",shift_len,"_subs_fc.pdf")
	cairo_pdf(fig_path, width = 12, height = 4)
	shift_subs_fc<-rep(0,nrow(del_fc))

	for(i in shift_len:1){
	  
	  if(i==shift_len){
		
		shift_fc_dn<-del_fc[,(i+10)]-del_fc[,i]
		shift_fc_dn[which(del_fc[,i] <=1 | shift_fc_dn<0)]<-0
		shift_fc_dn<-c(rep(0,i),shift_fc_dn[1:(aa_len+1-i)])
		
		barplot(shift_fc_dn,xpd=FALSE,width = bar_width,beside = F,
				col=rgb(255, 0, 0, 255/(i+1), maxColorValue=255),
				border="NA",ylim=c(0,y_lim),
				axes=F,xaxs = "i",space=space_value,cex.axis=1)
		
		shift_perfect_aa<-c(rep(0,i),del_fc[1:(aa_len+1-i),i])
		
		shift_fc_up<-del_fc[,(i+10)]-shift_perfect_aa
		shift_fc_up[which(shift_perfect_aa <=1 | shift_fc_up<0)]<-0
		
		barplot(shift_fc_up,xpd=FALSE,width = bar_width,beside = F,
				col=rgb(255, 0, 0, 255/(i+1), maxColorValue=255),
				border="NA",add=T,ylim=c(0,y_lim),
				axes=F,xaxs = "i",space=space_value)
		
		
		shift_subs_fc<-shift_subs_fc+shift_fc_dn+shift_fc_up
		
	  }else{
	  
	  print(i)
		
		shift_fc_dn<-del_fc[,(i+10)]-del_fc[,i]
		shift_fc_dn[which(del_fc[,i] <=1 | shift_fc_dn<0)]<-0
		shift_fc_dn<-c(rep(0,i),shift_fc_dn[1:(aa_len+1-i)])
		
		barplot(shift_fc_dn,xpd=FALSE,width = bar_width,beside = F,
				col=rgb(255, 0, 0, 255/(i+1), maxColorValue=255),
				border="NA",add=T,ylim=c(0,y_lim),
				axes=F,xaxs = "i",space=space_value)
		
		
		shift_perfect_aa<-c(rep(0,i),del_fc[1:(aa_len+1-i),i])
		
		shift_fc_up<-del_fc[,(i+10)]-shift_perfect_aa
		shift_fc_up[which(shift_perfect_aa <=1 | shift_fc_up<0)]<-0
		
		barplot(shift_fc_up,xpd=FALSE,width = bar_width,beside = F,
				col=rgb(255, 0, 0, 255/(i+1), maxColorValue=255),
				border="NA",add=T,ylim=c(0,y_lim),
				axes=F,xaxs = "i",space=space_value)
				
		shift_subs_fc<-shift_subs_fc+shift_fc_dn+shift_fc_up
		
	  }
	  
	}
	
	if(y_lim>1000){
	y_break<-round(y_lim/3,-2)
	}else(
	y_break=100
	)
	
	
	axis(side=1,labels =F,at=seq(0,axis_lim,axis_break))
	mtext(side=1,text=seq(0,axis_lim,axis_break),line=0.5,at=seq(0,axis_lim,axis_break),cex.lab=1,cex.axis=1)
	#axis(side=1,at=seq(0,ceiling(aa_len/20)*20,20),cex.lab=1,cex.axis=1)
	
	axis(side=2,at=seq(0,y_lim,y_break))
	dev.off()


	  write.table(shift_subs_fc, 
              paste0(tableFolder,"/",gene_label,"_subs_del_shift.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = T) 


	shift_subs_score<-norm_0_1(log2(shift_subs_fc+1))
#	fig_path<-paste0(figFolder,"/",gene_label,"_shift_",shift_len,"_subs_score_plot.png")
	fig_path<-paste0(figFolder,"/",gene_label,"_shift_",shift_len,"_subs_score_plot.pdf")
	#red_p<-c(115,127,141)
	
	if(aa_len>1000){
	black_p<-which(rank(-shift_subs_score)<11 & !c(1:length(shift_subs_score)%in%red_p))
	}else{
	black_p<-which(rank(-shift_subs_score)<5 & !c(1:length(shift_subs_score)%in%red_p))
	}
	
	print(black_p)
	#red_p<-NULL
	#black_p<-NULL
	plot_shift_subs_score(shift_subs_score,
	fig_path,red_p,black_p,WT_aa)		  

}


plot_shift_subs_score<-function(subs_score,fig_path,red_p,black_p,WT_aa){

	if(aa_len<300){
		axis_break<-20
	}else{
		axis_break<-100
	}

	#y_axis_lim<-ceiling(max(subs_score)/500)*500*1.1
	y_axis_lim<-1.2
	axis_lim<-axis_break*round(aa_len/axis_break,0)

#	png(fig_path, width =  1200, height = 400)
	cairo_pdf(fig_path, width =  12, height = 4)
	plot(c(1:length(subs_score)),subs_score,pch=16,col="grey",xpd=FALSE,axes = FALSE,
		 ylim=c(0,y_axis_lim),xlim=c(0,axis_break*round(aa_len/axis_break,0)),
		 #xlim=c(0,axis_break*ceiling(aa_len/axis_break)),
		 xaxs = "i",yaxs = "i",ann=FALSE)	
	axis(side=1,labels =F,at=seq(0,axis_lim,axis_break))
	mtext(side=1,text=seq(0,axis_lim,axis_break),line=0.5,at=seq(0,axis_lim,axis_break),cex.lab=1,cex.axis=1)
	axis(side=2,at=seq(0,y_axis_lim,0.2))

	########
	if(length(red_p)>0){
		points_index<-red_p
		points(points_index,subs_score[points_index],col="red",pch=16)
		with(data.frame(points_index,subs_score[points_index]),
		text(data.frame(points_index,subs_score[points_index]),
		labels=paste0(WT_aa[red_p],red_p),pos=c(2:4,1)))
	}

	if(length(black_p)>0){

		points_index<-black_p
		points(points_index,subs_score[points_index],col="black",pch=16)
		with(data.frame(points_index,subs_score[points_index]),
		text(data.frame(points_index,subs_score[points_index]),
		labels=paste0(WT_aa[black_p],black_p),pos=c(1,3,4,1,2)))

	}

	dev.off()



}




all_subs_type<-function(aa_matrix,subsize,outPath){

  if(is.null(subsize)){
  
    subsize<-(1:ncol(aa_matrix))
  }

  sub_aa_seq<-as.matrix(aa_matrix[,subsize])
  #aa_type_count<-as.numeric(apply(sub_aa_seq,2,function(x)(length(which(x>0)))))
  
  #aa_scale<-sub_aa_seq*1e04
  #aa_scale[which(aa_scale>1)]<-1
  #aa_scale<-log10(sub_aa_seq+1e-05)+5
  aa_scale<-sub_aa_seq
  #aa_scale[aa_scale>0]<-1
  
  aa_col<-function(aalist){
    ascii<-as.numeric(sapply(aalist, utf8ToInt))
    ascii-min(ascii)+1
    
  } 
  aa_len<-ncol(aa_matrix)
  	if(aa_len<300){
		axis_break<-20
	}else{
		axis_break<-100
	}
	
  if(aa_len>2000){
  
#    png(outPath, width = 3200, height = 600)
    cairo_pdf(outPath, width = 32, height = 6)	
  max_col<-min(max(aa_scale)*10*255,255)
  de_col<-rgb(255, 0, 0, c(1:max_col), maxColorValue=255)
  image2D(t(aa_scale),col=de_col,colkey = FALSE,
  border = "white",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)

  
  mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1,1/19), 
        las=1, cex=2,col = aa_col(colnames(t(sub_aa_seq))))
  mtext(side=1,text=seq(0,aa_len,axis_break),line=1,at=seq(0,1,axis_break/aa_len),cex=2)
  
  dev.off()
  
  
  }	else{
#    png(outPath, width = 1200, height = 300)
  cairo_pdf(outPath, width = 12, height = 3)  
  max_col<-min(max(aa_scale)*100*255,255)
  de_col<-rgb(255, 0, 0, c(1:max_col), maxColorValue=255)
  image2D(t(aa_scale),col=de_col,colkey = FALSE,
  border = "white",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)

  
  mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1,1/19), 
        las=1, cex=0.7,col = aa_col(colnames(t(sub_aa_seq))))
  mtext(side=1,text=seq(0,aa_len,axis_break),line=0.5,at=seq(0,1,axis_break/aa_len),cex.lab=1,cex.axis=1)
  
  dev.off()
  
  }


}

plot_int_shift_del<-function(figFolder,gene_label,sum_perf_del,sum_shift_del){

  fig_path<-paste0(figFolder,"/",gene_label,"_int_shift_del.pdf")
#  png(fig_path, width = 600, height = 600)
  cairo_pdf(fig_path, width = 10, height = 10)  
  cor_del<-round(cor(sum_perf_del,sum_shift_del),2)
  #plot_lims<-max(sum_perf_del,sum_shift_del)
  plot(sum_perf_del,
     sum_shift_del,pch=16,
     #pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="perfect aa deletions",ylab="shift aa deletions",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("two-type deletions of ",gene_label," (cor=",cor_del ,")"))
  #abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()


}

plot_del_subs_bar<-function(subs_ess,del_ess,aa_len,fig_path){

#	png(fig_path, width = 1200, height = 200)
	cairo_pdf(fig_path, width = 20, height = 4)	


	#space_multiply<-1
	if(aa_len<300){
		axis_break<-20
		bar_width<-0.6
	}else{
		axis_break<-100
		bar_width<-0.9
	}
	space_value<-(1-bar_width)/bar_width
	ess_singal<-rbind(del_ess,subs_ess)
	axis_lim<-axis_break*ceiling(aa_len/axis_break)
	#axis_lim<-axis_break*round(aa_len/axis_break,0)
	#axis_lim<-aa_len
		barplot(rbind(rep(0,length(del_ess)),del_ess),xpd=F,width = bar_width,beside = F,
			col=c(rgb(255, 255, 255, 0, maxColorValue=255),
			rgb(0, 0, 255, 120, maxColorValue=255)),
			border="NA",
			xlim=c(0,axis_lim),ylim=c(-1.2,1.2),
			axes=F,xaxs = "i",space=space_value,cex.axis=1)
	
	barplot(rbind(subs_ess,rep(0,length(subs_ess))),xpd=F,width = bar_width,beside = F,
	col=c(
	rgb(255, 0, 0, 120, maxColorValue=255),
	rgb(255, 255, 255, 0, maxColorValue=255)),
	border="NA",add=T,
	xlim=c(0,axis_lim),ylim=c(-1.2,1.2),
	axes=F,xaxs = "i",space=space_value,cex.axis=1)
	


	#barplot(subs_ess,xpd=F,width = bar_width,beside = TRUE,
	#		col=rgb(255, 0, 0, 120, maxColorValue=255),
	#		border="NA",
	#		xlim=c(0,axis_lim),
	#		axes=F,xaxs = "i",space=space_value,cex.axis=1)

	#barplot(del_ess,xpd=F,width = bar_width,beside = TRUE,
	#		xlim=c(0,axis_lim),
	#		col=rgb(0, 0, 255, 120, maxColorValue=255),
	#		border="NA",add=T,
	#		axes=F,xaxs = "i",space=space_value,cex.axis=1)
	axis(side=1,line=-5.4,labels =F,at=seq(0,axis_lim,axis_break))
	mtext(side=1,text=seq(0,axis_lim,axis_break),line=0.5,at=seq(0,axis_lim,axis_break),cex.lab=1,cex.axis=1)


	dev.off()

}


plot_pattern_ratio<-function(aa_matrix,subsize,WT_aa,highlight1=NULL,highlight2=NULL,outPath){

  sub_aa_seq<-as.matrix(aa_matrix[,subsize,drop=FALSE])
  #aa_type_count<-as.numeric(apply(sub_aa_seq,2,function(x)(length(which(x>0)))))
  
  #aa_scale<-sub_aa_seq*1e04
  #aa_scale[which(aa_scale>1)]<-1
  aa_scale<-sweep(sub_aa_seq,2,colSums(sub_aa_seq),"/")
  #aa_scale[which(aa_scale>0)]<-1
 
  
  label_col<-rep("dimgray",length(subsize))
  
  if(length(highlight1)>0){
    label_col[highlight1]<-"black"
  }
  
  if(length(highlight2)>0){
    label_col[highlight2]<-"red"
  }
  
  if(length(subsize)>30){
#    png(outPath, width = max(50*length(subsize),300), height = 500)
	cairo_pdf(outPath)
  }else{
#    png(outPath, width = max(50*length(subsize),300), height = 500)
	cairo_pdf(outPath)
  }

#  plot(raster(aa_scale,xmn=0, xmx=length(subsize), ymn=0, ymx=20, crs=NA, template=NULL),
#  col=c("white","lightblue"),key=NULL,
#  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE,
#  xaxt='n',yaxt='n', ylab="", xlab="") 
 # image2D(t(aa_scale),col=c("#FFFFFF","#0000FF"),colkey = T,breaks =50,
 de_col<-rgb(255, 0, 0, c(1:255), maxColorValue=255)
 #print(aa_scale)
 if(length(subsize)>1){
   image2D(t(aa_scale),col=de_col,
   colkey = list(dist = 0,length = 0.5, width = 0.6,
   cex.axis = 0.7,tck=-0.02,col.box = "white"),
  border = "gray90",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)
 }else{
   image2D(t(aa_scale),col=de_col,colkey = T,
  border = "white",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)
 }

  #plot(raster(aa_scale))
  #image(t(aa_scale), col=c("white","lightblue"),axes=F,oldstyle=T)
  #mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1.05,0.053), 
  #      las=1, cex=1,col = aa_col(colnames(t(sub_aa_seq))))
  #print(rownames(sub_aa_seq))
  mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1,1/19), 
        las=1, cex=1,col = "black")
  mtext(text=paste0(WT_aa[subsize],subsize), side=1, line=0.3, at=seq(0,1,1/(length(subsize)-1)), 
        las=1, cex=1,padj=0, col =label_col)
  
  dev.off()
  
  
}




plot_mutation_pattern<-function(aa_matrix,subsize,WT_aa,highlight1=NULL,highlight2=NULL,outPath){

  sub_aa_seq<-as.matrix(aa_matrix[,subsize,drop=FALSE])
  aa_type_count<-as.numeric(apply(sub_aa_seq,2,function(x)(length(which(x>0)))))
  
  #aa_scale<-sub_aa_seq*1e04
  #aa_scale[which(aa_scale>1)]<-1
  aa_scale<-sub_aa_seq
  #aa_scale[which(aa_scale>0)]<-1
  
  aa_col<-function(aalist){
    ascii<-as.numeric(sapply(aalist, utf8ToInt))
    ascii-min(ascii)+1
    
  } 
  
  label_col<-rep("dimgray",length(subsize))
  
  if(length(highlight1)>0){
    label_col[highlight1]<-"black"
  }
  
  if(length(highlight2)>0){
    label_col[highlight2]<-"red"
  }
  
  if(length(subsize)>30){
#    png(outPath, width = max(50*length(subsize),300), height = 500)
	cairo_pdf(outPath)
  }else{
#    png(outPath, width = max(50*length(subsize),300), height = 500)
	cairo_pdf(outPath)
  }

#  plot(raster(aa_scale,xmn=0, xmx=length(subsize), ymn=0, ymx=20, crs=NA, template=NULL),
#  col=c("white","lightblue"),key=NULL,
#  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE,
#  xaxt='n',yaxt='n', ylab="", xlab="") 
 # image2D(t(aa_scale),col=c("#FFFFFF","#0000FF"),colkey = T,breaks =50,
 de_col<-rgb(255, 0, 0, c(1:255), maxColorValue=255)
 #print(aa_scale)
 if(length(subsize)>1){
   image2D(t(aa_scale),col=de_col,colkey = F,
  border = "white",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)
 }else{
   image2D(t(aa_scale),col=de_col,colkey = F,
  border = "white",
  axes=FALSE, xaxs = "i",yaxs = "i",ann=FALSE,xpd=FALSE)
 }

  #plot(raster(aa_scale))
  #image(t(aa_scale), col=c("white","lightblue"),axes=F,oldstyle=T)
  #mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1.05,0.053), 
  #      las=1, cex=1,col = aa_col(colnames(t(sub_aa_seq))))
  #print(rownames(sub_aa_seq))
  mtext(text=colnames(t(sub_aa_seq)), side=2, line=0.3, at=seq(0,1,1/19), 
        las=1, cex=1,col = "black")
  mtext(text=paste0(WT_aa[subsize],subsize), side=1, line=0.3, at=seq(0,1,1/(length(subsize)-1)), 
        las=1, cex=1,padj=0, col =label_col)
  
  dev.off()
  
  
}


plot_del_lfc<-function(del_lfc,aa_len,del_cut){
  
  barplot(del_lfc[,"single_aa_del"],space=0,ylim=c(del_cut,ceiling(max(del_lfc))),xlim=c(0,20*ceiling(aa_len/20)),
          border=NA,col="blue",cex.lab=2,cex.axis = 2,
          xpd=FALSE,beside=TRUE,xaxs = "i")
  axis(side=1,at=seq(0,20*ceiling(aa_len/20),20),cex.lab=2,cex.axis = 2)
  print(which(del_lfc[,"single_aa_del"]>del_cut))
  
  del_lfc<-del_lfc[,c(1:(ncol(del_lfc)-1))]
  for(aa in 1:nrow(del_lfc)){
    del_len<-which(del_lfc[aa,]>0)
    
    if(length(del_len)>0){
      #print(aa)
      
      for(p in del_len){
        bar<-rep(0,aa_len)
        ### bar_length is del_length+1
        bar[aa:c(aa+p)]<-del_lfc[aa,p]
		
		barplot(bar,space=0,add=T,
		border=NA,cex.axis=2,
		xpd=FALSE,beside=TRUE,xaxs = "i",
		col=rgb(0, 0, 255, 40, maxColorValue=255))
        
      }
      
    }
    
  }
  legend("topright",
         c("single aa deletion","multiple aa deletion"),
         cex=2,
         fill = c("blue",
                  rgb(0, 0, 255, 40, maxColorValue=255)),
				  border = "white", bty = 'n')
  
  
}

## plot deletion 
plot_del_lfc_transp<-function(del_lfc,aa_len,del_cut,del_len_cut,del_type){

para_lwd<-1
para_cex_lab<-1
para_cex_axis<-1

bar_width<-0.85
para_space<-(1-bar_width)/bar_width
#para_space<-0.182

del_lfc[which(del_lfc==1)]<-0

	if(aa_len<300){
		axis_break<-20
	}else if(aa_len<700){
		axis_break<-100
	}else if(aa_len<700){
		axis_break<-60
	}else if(aa_len<1000){
		axis_break<-100
	}else{
		axis_break<-200
	}
	#axis_lim<-aa_len
	axis_lim<-axis_break*ceiling(aa_len/axis_break)

	#
  #print(which(del_lfc[,"single_aa_del"]>del_cut))
  #single_aa_del<-del_lfc[,"single_aa_del"]
  single_aa_del<-del_lfc[,1]
  if(del_len_cut>0){
    y_lim<-ceiling(max(del_lfc[,c(1:del_len_cut)],single_aa_del))
  }else{
    y_lim<-ceiling(max(single_aa_del))
  }

  if(y_lim>100){
  
  y_lim<-ceiling(y_lim/100)*100
  }
  if(y_lim>1000){ 
  y_lim<-ceiling(y_lim/1000)*1000
  }
  
    
  if(y_lim>5000){ 
  y_lim<-ceiling(y_lim/5000)*5000
  }
  
#  if(y_lim>10000){ 
#  y_lim<-ceiling(y_lim/10000)*10000
#  }
#print(y_lim)
  if(del_len_cut>0){
  
	del_lfc<-del_lfc[,c(1:del_len_cut),drop=FALSE]
	flag<-1
	  for(aa in 1:nrow(del_lfc)){
		del_len<-which(del_lfc[aa,]>0)
		
		if(length(del_len)>0){
		  #print(aa)
		  
		  for(ii in 1:length(del_len)){
			bar<-rep(0,aa_len)
			#print(ii)
			
			p<-del_len[ii]

			if(del_type==1){
			  #bar_col<-rgb(0, 0, 255, 180/p, maxColorValue=255)
			  bar_col<-rgb(0, 0, 255, 38, maxColorValue=255)
			}else{
			#bar_col<-rgb(128,0,128, 180/p, maxColorValue=255)
			  bar_col<-rgb(200,0,200, 255/(p+1), maxColorValue=255)
			}
			
			if(del_type==1){
			  bar[aa:c(aa+p-1)]<-del_lfc[aa,p]
			}else{
			  bar[aa:c(aa+p)]<-del_lfc[aa,p]
			}
			
			
			
			
			if(flag==1){

			### 3nt deletion cover two aa
#print("in")
			barplot(bar,space=para_space,width=bar_width,
xlim=c(-0.4,axis_lim),ylim=c(del_cut,y_lim),
			border=NA,tcl = NA,xaxs = "i",yaxs = "i",
			xpd=FALSE,beside=TRUE,
			col=bar_col)

			flag<-0
			
			}else{
			### bar_length is del_length
			#bar[aa:c(aa+p-1)]<-del_lfc[aa,p]

			barplot(bar,space=para_space,width=bar_width,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,
			xpd=FALSE,beside=TRUE,
			col=bar_col)
			}

		  }
		  
		}
		
	  }
  
  
  
  }
  
if(del_type==1){
  bar_col<-rgb(0, 0, 255, 255, maxColorValue=255)

	if(del_len_cut==0){
		barplot(single_aa_del,space=para_space,width=bar_width,
		xlim=c(-0.4,axis_lim),ylim=c(del_cut,y_lim),tcl = NA,
			  border=NA,col=bar_col,xaxs = "i",axes=F,
			  xpd=FALSE,beside=TRUE,xlab="",ylab="")
		
	}else{
		barplot(single_aa_del,space=para_space,width=bar_width,
		add=T,tcl = NA,xaxs = "i",axes=F,
			  border=NA,col=bar_col,
			  xpd=FALSE,beside=TRUE,xlab="",ylab="")

	}


}

axis(side=1,
		at=seq(-0.4,axis_lim,axis_break),label=seq(0,axis_lim,axis_break),tck=-0.02)

#  legend("topright",
#         c("single aa deletion","2 aa deletion",
#		 "3 aa deletion","4 aa deletion","5 aa deletion",
#		 "6 aa deletion","7 aa deletion","8 aa deletion",
#		 "9 aa deletion"),
#         cex=0.8,
#         fill = c("darkblue",
#                  rgb(0, 0, 255, 255/2, maxColorValue=255),
#				  rgb(0, 0, 255, 255/3, maxColorValue=255),
#				  rgb(0, 0, 255, 255/4, maxColorValue=255),
#				  rgb(0, 0, 255, 255/5, maxColorValue=255),
#				  rgb(0, 0, 255, 255/6, maxColorValue=255),
#				  rgb(0, 0, 255, 255/7, maxColorValue=255),
#				  rgb(0, 0, 255, 255/8, maxColorValue=255),
#				  rgb(0, 0, 255, 255/9, maxColorValue=255)),
#				  border = NA, bty = 'n')
  

}




## plot deletion 
discarded_plot_del_lfc_transp_double<-function(del_lfc,aa_len,del_cut,del_len_cut){

para_lwd<-1
para_cex_lab<-1
para_cex_axis<-1

bar_width<-0.85
para_space<-(1-bar_width)/bar_width

#para_space<-0

if(aa_len<300){
	axis_break<-20
}else{
	axis_break<-100
}

axis_lim<-axis_break*ceiling(aa_len/axis_break)

single_aa_del<-del_lfc[,1]

if(del_len_cut>0){
	y_lim<-ceiling(max(del_lfc,single_aa_del))
	}else{
	y_lim<-ceiling(max(single_aa_del))
}
#print(y_lim)
  if(del_len_cut>0){
  
	del_lfc1<-del_lfc[,c(1:del_len_cut),drop=FALSE]
	del_lfc2<-del_lfc[,c(11:(10+del_len_cut)),drop=FALSE]
	flag<-1
	  for(aa in 1:nrow(del_lfc1)){
		del_len<-which(del_lfc1[aa,]>0)
		
		if(length(del_len)>0){
		  #print(aa)
		  
		  for(ii in 1:length(del_len)){
			bar1<-bar2<-rep(0,aa_len)
			#print(ii)
			p<-del_len[ii]
			
			bar_col1<-rgb(0, 0, 255, 180/p, maxColorValue=255)
			#bar_col2<-rgb(128,0,128, 180/p, maxColorValue=255)
			bar_col2<-rgb(255,255,0, 180/p, maxColorValue=255)
			
			bar1[aa:c(aa+p-1)]<-del_lfc1[aa,p]
			bar2[aa:c(aa+p)]<-del_lfc2[aa,p]
			
			if(flag==1){

			### 3nt deletion cover two aa

			barplot(bar1,space=para_space,width=bar_width,
xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),
			border=NA,tcl = NA,xaxs = "i",yaxs = "i",
			xpd=FALSE,beside=TRUE,
			col=bar_col1)

			flag<-0
			
			}else{
			### bar_length is del_length
			#bar[aa:c(aa+p-1)]<-del_lfc[aa,p]

			barplot(bar1,space=para_space,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,width=bar_width,
			xpd=FALSE,beside=TRUE,
			col=bar_col1)
			
			barplot(bar2,space=para_space,add=T,tcl = NA,
			border=NA,xaxs = "i",axes=F,width=bar_width,
			xpd=FALSE,beside=TRUE,
			col=bar_col2)
			} ### flag if			
		  }### for del_len		  
		}## del_len if		
	  } ## for aa position 
  }### del_len_cut if
  

  bar_col1<-rgb(0, 0, 255, 240, maxColorValue=255)
#  bar_col2<-rgb(128,0,128, 240, maxColorValue=255)
  
  if(del_len_cut==0){
    barplot(single_aa_del,space=para_space,tcl = NA,width=bar_width,
	xlim=c(0,axis_lim),ylim=c(del_cut,y_lim),
          border=NA,col=bar_col1,xaxs = "i",axes=F,
          xpd=FALSE,beside=TRUE,xlab="",ylab="")
   axis(side=1,at=seq(0,axis_lim,axis_break),tck=-0.02)
  }else{
    barplot(single_aa_del,space=para_space,add=T,tcl = NA,width=bar_width,
          border=NA,col=bar_col1,xaxs = "i",axes=F,
          xpd=FALSE,beside=TRUE,xlab="",ylab="")
   axis(side=1,at=seq(0,axis_lim,axis_break),tck=-0.02)
  
  }

  

#  legend("topright",
#         c("single aa deletion","2 aa deletion",
#		 "3 aa deletion","4 aa deletion","5 aa deletion",
#		 "6 aa deletion","7 aa deletion","8 aa deletion",
#		 "9 aa deletion"),
#         cex=0.8,
#         fill = c("darkblue",
#                  rgb(0, 0, 255, 255/2, maxColorValue=255),
#				  rgb(0, 0, 255, 255/3, maxColorValue=255),
#				  rgb(0, 0, 255, 255/4, maxColorValue=255),
#				  rgb(0, 0, 255, 255/5, maxColorValue=255),
#				  rgb(0, 0, 255, 255/6, maxColorValue=255),
#				  rgb(0, 0, 255, 255/7, maxColorValue=255),
#				  rgb(0, 0, 255, 255/8, maxColorValue=255),
#				  rgb(0, 0, 255, 255/9, maxColorValue=255)),
#				  border = NA, bty = 'n')
  

}


save_plot_del_lfc_two_type<-function(del_lfc,aa_len,del_cut,fig_path,del_len_cut){
  #plot_del_lfc_transp(del_lfc,aa_len,del_cut)
#  png(fig_path, width = 1200, height = 300)
cairo_pdf(paste0(fig_path,".pdf"), width = 16, height = 4) 
#    par(mar=c(0,0,0,0)+0.5)
  #plot_del_lfc(del_lfc,aa_len,del_cut)
  plot_del_lfc_transp_double(del_lfc,aa_len,del_cut,del_len_cut)
  dev.off()
}



save_plot_del_lfc<-function(del_lfc,aa_len,del_cut,fig_path,del_len_cut,del_type){
  #plot_del_lfc_transp(del_lfc,aa_len,del_cut)
#  png(fig_path, width = 1200, height = 300)
#  pdf(fig_path, width = 6, height = 1,compress=FALSE) 
  cairo_pdf(paste0(fig_path,".pdf"), width = 16, height = 4) 
 # cairo_pdf(paste0(fig_path,".pdf"), width = 10, height = 8) 
  #plot_del_lfc(del_lfc,aa_len,del_cut)
  #par(mar=c(0,0,0,0)+0.5)

  plot_del_lfc_transp(del_lfc,aa_len,del_cut,del_len_cut,del_type)
  dev.off()
}



plot_del_score<-function(del_score,fig_path,red_p,black_p,WT_aa){

aa_len<-length(WT_aa)
	if(aa_len<300){
		axis_break<-20
	}else if(aa_len<700){
		axis_break<-100
	}else if(aa_len<700){
		axis_break<-60
	}else if(aa_len<1000){
		axis_break<-100
	}else{
		axis_break<-200
	}

	y_axis_lim<-ceiling(max(del_score[,2])*1.1)
    y_break<-max(floor(y_axis_lim/4),1)
	#print(y_axis_lim)
	#png(fig_path, width = 1200, height = 300)
    cairo_pdf(fig_path, width = 16, height = 4)
	plot(del_score[,1:2],pch=16,col="grey",xpd=FALSE,axes = FALSE,
	#xlim=c(0,aa_len),
		 ylim=c(0,y_axis_lim),
		 #xlim=c(0,axis_break*round(aa_len/axis_break,0)),
		 xlim=c(0,axis_break*ceiling(aa_len/axis_break)),
		 xaxs = "i",yaxs = "i",ann=FALSE)
	abline(h = 1, lty=2)
	axis(side=1,at=seq(0,axis_break*ceiling(aa_len/axis_break),axis_break),tck=-0.02)
	#axis(side=1,at=seq(0,aa_len,axis_break))
	if(y_axis_lim>4){
	axis(side=2,at=seq(0,y_axis_lim,y_break),tck=-0.02)
	}else{
	axis(side=2,at=seq(0,y_axis_lim,y_break),tck=-0.02)
	}
	

	########
	
	if(length(black_p)>0){

	points_index<-black_p
	points(del_score[points_index,1:2],col="black",pch=16)
	#with(del_score[points_index,1:2],text(del_score[points_index,1:2],labels=paste0(WT_aa[black_p],black_p),pos=c(3,4,1)))

}
	if(length(red_p)>0){
		points_index<-red_p
		points(del_score[points_index,1:2],col="red",pch=16)
		#with(del_score[points_index,1:2],text(del_score[points_index,1:2],labels=paste0(WT_aa[red_p],red_p),pos=c(2,3,4)))
	}



	dev.off()



}





