####### subs_function.R
library(plyr)
library("plotrix")
library("plot.matrix")
library(raster)  
library(plot3D)  




## calculate fold-change of deletion
output_del<-function(folder,cds_start,cds_end,count_th,ref1_label,gene_label,singleDelWeight,del_cut,hist_ylim,del_len_cut){
  
  figFolder<-paste0("results_fig_",gene_label)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",gene_label)
  if(!dir.exists(tableFolder))dir.create(tableFolder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))
  aa_len<-nchar(aa_seq)
  
  ##----
  codon_depth_wt<-as.matrix(read.table(paste0(folder,"/",wt_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  wt_file<-read.table(paste0(folder,"/",wt_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(wt_file[,3]))
  
  wt_file<-wt_file[which(as.numeric(wt_file[,3])>count_th),]
  #del_codon_wt<-convert_nt_to_codon(wt_file,cds_start,cds_end)
  
  del_codon_wt<-convert_nt_to_codon(wt_file,cds_start,cds_end)
  del_matrix_wt<-del_freq_from_count(del_codon_wt,codon_depth_wt)
  del_freq_wt<-c(as.matrix(del_matrix_wt[,4]))

  
  codon_depth_ref1<-as.matrix(read.table(paste0(folder,"/",ref1_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  ref1_file<-read.table(paste0(folder,"/",ref1_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(ref1_file[,3]))
  
  ref1_file<-ref1_file[which(as.numeric(ref1_file[,3])>count_th),]
  del_codon_ref1<-convert_nt_to_codon(ref1_file,cds_start,cds_end)
  del_matrix_ref1<-del_freq_from_count(del_codon_ref1,codon_depth_ref1)
  #del_freq_ref1<-c(as.matrix(del_matrix_ref1[,4]))
  del_freq_matrix_ref1<-construct_del_matrix(del_matrix_ref1,aa_len)
  del_freq_ref1<-c(as.matrix(del_freq_matrix_ref1))
  
  write.table(del_freq_matrix_ref1, 
              paste0(tableFolder,"/",gene_label,"_del_ref_generate.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = TRUE) 
   
   
#### optional---- WT data process and plot ------  
  del_freq_matrix_wt<-construct_del_matrix(del_matrix_wt,aa_len)
  del_freq_wt<-c(as.matrix(del_freq_matrix_wt))
  #summary(del_freq_wt)
  

  
  png(paste0(figFolder,"/",gene_label,"_del_freq_ref_wt.png"), width = 600, height = 600)
  plot(log10(del_freq_wt+1e-07),log10(del_freq_ref1+1e-07),
       col=rgb(0, 0, 255, 120, maxColorValue=255),
	   pch=16,main=gene_label,
       xlab="del_freq_wt",ylab="del_freq_ref")
  abline(a = 0, b = 1, col = 2,pch=2)
  dev.off()

  png(paste0(figFolder,"/",gene_label,"_hist_ref_wt.png"), width = 600, height = 600)
  hist(log10(del_freq_matrix_ref1+1e-07),breaks=20,ylim=c(0,hist_ylim),
       col=rgb(0, 0, 255, 60, maxColorValue=255),
       main=paste0("Deletion frequency of ",gene_label))
  
  hist(log10(del_freq_matrix_wt+1e-07),ylim=c(0,hist_ylim),breaks=20,
       col=rgb(255, 255, 0, 60, maxColorValue=255),add=T)
  legend("topright",
         c("D14_ref1","WT"),
         fill = c(rgb(0, 0, 255, 60, maxColorValue=255),
                  rgb(255, 255, 0, 60, maxColorValue=255)),
				  border = "white",bty = 'n')
  
  dev.off()
  #### optional---- WT data process and plot over ------  
  
}



## output substituion info
output_subs<-function(folder,cds_start,cds_end,count_th,ref1_label,wt_label,gene_label){
  
  figFolder<-paste0("results_fig_",gene_label)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",gene_label)
  if(!dir.exists(tableFolder))dir.create(tableFolder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))
  aa_len<-nchar(aa_seq)
  
  subs_ref1_freq<-subs_count_cut_freq(folder,ref1_label,count_th)  
  
    subs_wt_freq<-subs_count_cut_freq(folder,wt_label,count_th)
	
######	wt codon
  ref_path<-paste0("aa_seq/",gene_label,".fa")
   ref_seq<-readDNAStringSet(ref_path)
  cds_seq<-subseq(ref_seq, start=cds_start+1, end=cds_end)
  wt_codon<-as.character(codons(cds_seq[[1]]))
  #print(wt_codon)
  for(pos in 1:length(wt_codon)){
    subs_ref1_freq[wt_codon[pos],pos]<-0
	subs_wt_freq[wt_codon[pos],pos]<-0
  }
##########  
  
  ### remove SNP 
  SNP_cut<-0.1
  tmp<-which(subs_ref1_freq>SNP_cut)
  if(length(tmp)>0){
  print("SNP")
  print(tmp)

	  subs_ref1_freq[subs_ref1_freq>SNP_cut]<-0

	  subs_wt_freq[subs_ref1_freq>SNP_cut]<-0
  }
	
	
  aa_subs_ref1_freq<-codonToAA(subs_ref1_freq)

  aa_subs_wt_freq<-codonToAA(subs_wt_freq)
  
  ref1_sum_freq<-colSums(aa_subs_ref1_freq)

   
  aa_ref_freq<-aa_subs_ref1_freq
  
  
  wt_subs_aa_list<-c(as.matrix(aa_subs_wt_freq))
  ref_subs_aa_list<-c(as.matrix(aa_ref_freq))
  
  log_wt_subs_aa_list<-log10(wt_subs_aa_list+1e-08)
  log_ref_subs_aa_list<-log10(ref_subs_aa_list+1e-08)
  
  ###  optional--- WT process------
  fig_path<-paste0(figFolder,"/",gene_label,"_hist_log_subs_aa_wt_ref.png")
  png(fig_path, width = 600, height = 600)  
    hist(log_ref_subs_aa_list,breaks=30,
       xlim=c(-7,0),ylim=c(1,100),
       col=rgb(0, 0, 255, 60, maxColorValue=255),
       main=paste0("subs_aa_wt_ref_",gene_label))
  
  hist(log_wt_subs_aa_list,breaks=30,
       col=rgb(0, 255, 255, 60, maxColorValue=255),add=T)
	   
    legend("topright",
         c("reference","WT"),
         fill = c(rgb(0, 0, 255, 60, maxColorValue=255),
                  rgb(0, 255, 255, 60, maxColorValue=255)),
				  border = "white",bty = 'n')
  dev.off()
  
  #####
  
  wt_subs_aa_list<-wt_subs_aa_list[which(ref_subs_aa_list>0)]
  ref_subs_aa_list<-ref_subs_aa_list[which(ref_subs_aa_list>0)]
  
  log_wt_subs_aa_list<-log10(wt_subs_aa_list+1e-08)
  log_ref_subs_aa_list<-log10(ref_subs_aa_list+1e-08)
  
  
  fig_path<-paste0(figFolder,"/",gene_label,"_log_subs_aa_wt_ref.png")
  png(fig_path, width = 600, height = 600)
  
  plot_lims<-max(log_wt_subs_aa_list,log_ref_subs_aa_list)
  plot(log_wt_subs_aa_list,
     log_ref_subs_aa_list,
     pch=16,xlim=c(-8,plot_lims),ylim=c(-8,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
  fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref.png")
  png(fig_path, width = 600, height = 600)
  
  plot_lims<-max(wt_subs_aa_list,ref_subs_aa_list)
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
      fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref_low03.png")
  png(fig_path, width = 600, height = 600)
  
  #plot_lims<-max(c(as.matrix(aa_subs_wt_freq)),c(as.matrix(aa_ref_freq)))
  plot_lims<-1e-02
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
  
    fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref_low04.png")
  png(fig_path, width = 600, height = 600)
  
  #plot_lims<-max(c(as.matrix(aa_subs_wt_freq)),c(as.matrix(aa_ref_freq)))
  plot_lims<-1e-03
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
  fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref_low05.png")
  png(fig_path, width = 600, height = 600)
  
  #plot_lims<-max(c(as.matrix(aa_subs_wt_freq)),c(as.matrix(aa_ref_freq)))
  plot_lims<-1e-04
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
    fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref_low06.png")
  png(fig_path, width = 600, height = 600)
  
  #plot_lims<-max(c(as.matrix(aa_subs_wt_freq)),c(as.matrix(aa_ref_freq)))
  plot_lims<-1e-05
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()

  
      fig_path<-paste0(figFolder,"/",gene_label,"_subs_aa_wt_ref_low07.png")
  png(fig_path, width = 600, height = 600)
  
  #plot_lims<-max(c(as.matrix(aa_subs_wt_freq)),c(as.matrix(aa_ref_freq)))
  plot_lims<-1e-06
  plot(wt_subs_aa_list,
     ref_subs_aa_list,
     pch=16,xlim=c(0,plot_lims),ylim=c(0,plot_lims),
     xlab="wt aa freq",ylab="ref aa freq",
     col=rgb(0, 0, 255, 120, maxColorValue=255),
	 main=paste0("single aa substitution of",gene_label))
  abline(a = 0, b = 1, col = 2,pch=2)

  dev.off()
  
  write.table(aa_ref_freq, 
              paste0(tableFolder,"/",gene_label,"_subs_aa_ref_freq.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = TRUE,col.names = NA) 
 
  
  aa_subs_generate<-matrix(0,ncol=ncol(aa_ref_freq),nrow=nrow(aa_ref_freq))
  aa_subs_generate[which(aa_ref_freq>1e-06)]<-1
  
  print(summary(colSums(aa_subs_generate)))
    write.table(colSums(aa_subs_generate), 
              paste0(tableFolder,"/",gene_label,"_subs_generate_count.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = TRUE,col.names = NA) 
			  
	write.table(aa_subs_generate, 
		  paste0(tableFolder,"/",gene_label,"_subs_generate_type.csv"), 
		  append = FALSE, quote = FALSE, sep = ",",
		  row.names = TRUE,col.names = NA) 
   
  
}







